set.seed(3534)
# Read in file & library ----
library(tidyverse)
library(genefu)
library(survival)
library(nhs.predict)
library(mclust)
library(survcomp)
library(ggalt)
library(rms)
outcomes <- read_csv("brca_patient_outcomes.csv") %>%

  # z-normalise EP score
  mutate(across(.cols = "EP", .fns = ~scale(.)))

# Cox PH models of association----

# Define genomic scores
models <- c("ODx", "EP", "MP", "ROR")

# Define surv object for BCSS in ER+/ER- patients separately
erplus <- filter(outcomes, ER.Expr == 1) %>%
  drop_na(PREDICT, all_of(models), DeathBreast, `T`)
erminus <- filter(outcomes, ER.Expr == 0) %>%
  drop_na(PREDICT, all_of(models), DeathBreast, `T`)

survplus <- with(erplus, Surv(`T`, DeathBreast))
survminus <-  with(erminus, Surv(`T`, DeathBreast))

# Produce univariable models for BCSS in ER+
pi <- summary(coxph(survplus ~ offset(PREDICT), data = erplus))

univar <- lapply(erplus[, models],
                 function(x) coxph(survplus ~ x, data = erplus))

## Now with bootstrap
boot <- NULL
for (m in models) {
    n <- nrow(erplus)
    B <- 100
    C.tr <- numeric(B)
    C.tt <- numeric(B)
    dd <- erplus[,c('T', 'DeathBreast', 'PREDICT', m)]
    colnames(dd)[4] <- "x"
    for (i in 1:B) {
        sb <- sample(1:n, replace = TRUE)
        m.b <- coxph(Surv(T, DeathBreast) ~ x,
                     data=dd[sb,])
        C.tr[i] <- summary(m.b)$concordance['C']
        C.tt[i] <- I(-dxy.cens(predict(m.b, newdata=dd[-sb,]),
                               Surv(dd$T[-sb],
                                    dd$DeathBreast[-sb]))/2 + 0.5)['Dxy']
    }
    boot[m] <- mean(C.tr-C.tt)
}


# Extract and store model results
summaries <- lapply(univar, summary)
univar_erplus <- as_tibble(cbind(

  # Populate with info from summaries
  model = models,
  coefs = sapply(summaries, function(x) x$coefficients[1]),
  loglik = sapply(summaries, function(x) x$loglik[2]),
  ses = sapply(summaries, function(x) x$coefficients[1,3]),
  pvals = sapply(summaries, function(x) x$coefficients[1,5]),
  c_index = sapply(summaries, function(x) x$concordance[1]),
  c_ind_se = sapply(summaries, function(x) x$concordance[2]),
  c_ind_optim = boot)) %>%

  # Convert to numeric
  mutate(across(.cols = c(2:8), .fns = ~ as.numeric(.))) %>%

  # Append PREDICT separately due to difference in model summary
  add_row(model = "PREDICT",
          coefs = 1,
          loglik = pi$loglik,
          ses = NA_integer_,
          pvals = NA_integer_,
          c_index = pi$concordance[6],
          c_ind_se = pi$concordance[7],
          c_ind_optim= NA_integer_) %>%

  # Neaten up appearance and calculate CIs
  transmute(model = model,
            coefficients = signif(coefs, 3),
            hr = signif(exp(coefs), 3),
            log_likelihood = signif(loglik, 5),
            standard_error = signif(ses,3),
            hr_lci = signif(hr - qnorm(.975) * ses, 3),
            hr_uci = signif(hr + qnorm(.975) * ses, 3),
            p_value = signif(pvals, 3),
            concordance_index = signif(c_index, 3),
            c_index_lci = signif(c_index - qnorm(.975) * c_ind_se, 3),
            c_index_uci = signif(c_index + qnorm(.975) * c_ind_se, 3),
            c_ind_optim=signif(c_ind_optim, 3))

rm(summaries, univar, pi)

# Produce univariable models for BCSS in ER-
pi <- summary(coxph(survminus ~ offset(PREDICT), data = erminus))

univar <- lapply(erminus[, models],
                 function(x) coxph(survminus ~ x, data = erminus))

## Now with bootstrap
boot <- NULL
for (m in models) {
    n <- nrow(erminus)
    C.tr <- numeric(B)
    C.tt <- numeric(B)
    B <- 100
    dd <- erminus[,c('T', 'DeathBreast', 'PREDICT', m)]
    colnames(dd)[4] <- "x"
    for (i in 1:B) {
        sb <- sample(1:n, replace = TRUE)
        m.b <- coxph(Surv(T, DeathBreast) ~ x,
                     data=dd[sb,])
        C.tr[i] <- summary(m.b)$concordance['C']
        C.tt[i] <- I(-dxy.cens(predict(m.b, newdata=dd[-sb,]),
                               Surv(dd$T[-sb],
                                    dd$DeathBreast[-sb]))/2 + 0.5)['Dxy']
    }
    boot[m] <- mean(C.tr-C.tt)
}

                                        # Extract and store model results
summaries <- lapply(univar, summary)
univar_erminus <- as_tibble(cbind(

  # Populate with info from summaries
  model = models,
  coefs = sapply(summaries, function(x) x$coefficients[1]),
  loglik = sapply(summaries, function(x) x$loglik[2]),
  ses = sapply(summaries, function(x) x$coefficients[1,3]),
  pvals = sapply(summaries, function(x) x$coefficients[1,5]),
  c_index = sapply(summaries, function(x) x$concordance[1]),
  c_ind_se = sapply(summaries, function(x) x$concordance[2]),
    c_ind_optim = boot)) %>%

  # Convert to numeric
  mutate(across(.cols = c(2:8), .fns = ~ as.numeric(.))) %>%

  # Append PREDICT separately due to difference in model summary
  add_row(model = "PREDICT",
          coefs = 1,
          loglik = pi$loglik,
          ses = NA_integer_,
          pvals = NA_integer_,
          c_index = pi$concordance[6],
          c_ind_se = pi$concordance[7],
          c_ind_optim = NA_integer_) %>%

  # Neaten up appearance and calculate CIs
  transmute(model = model,
            coefficients = signif(coefs, 3),
            hr = signif(exp(coefs), 3),
            log_likelihood = signif(loglik, 5),
            standard_error = signif(ses,3),
            hr_lci = signif(hr - qnorm(.975) * ses, 3),
            hr_uci = signif(hr + qnorm(.975) * ses, 3),
            p_value = signif(pvals, 3),
            concordance_index = signif(c_index, 3),
            c_index_lci = signif(c_index - qnorm(.975) * c_ind_se, 3),
            c_index_uci = signif(c_index + qnorm(.975) * c_ind_se, 3),
            c_ind_optim =signif(c_ind_optim, 3))

rm(summaries, univar, pi)

# Produce single table of univariable models and write to .csv
univar <- bind_rows(univar_erplus, univar_erminus, .id = "ER") %>%
  mutate(ER = case_when(ER == 1 ~ "Positive", ER == 2 ~ "Negative")) %>%

  # Tidy for final table
  transmute(
    ER = ER,
    Model = model,
    "HR (95%CI)" = paste(hr, " (", hr_lci, "-", hr_uci, ")", sep = ""),
    "Log-likelihood" = log_likelihood,
    "c-index (95%CI)" = paste(concordance_index, " (", c_index_lci, "-", c_index_uci, ")", sep = ""),
    "c-index optimism"=c_ind_optim)

write_csv(univar, "univariable_coxph.csv")

# Calculation of multivariable models in ER+ patients
piplus <- summary(coxph(survplus ~ offset(PREDICT), data = erplus))

multiplus <- lapply(erplus[, models],
                    function(x) coxph(survplus ~ offset(PREDICT) + x,
                                      data = erplus))

## bootstrap
boot <- NULL
for (m in models) {
    n <- nrow(erplus)
    B <- 100
    C.tr <- numeric(B)
    C.tt <- numeric(B)
    dd <- erplus[,c('T', 'DeathBreast', 'PREDICT', m)]
    colnames(dd)[4] <- "x"
    for (i in 1:B) {
        sb <- sample(1:n, replace = TRUE)
        m.b <- coxph(Surv(T, DeathBreast) ~ offset(PREDICT) + x,
                     data=dd[sb,])
        C.tr[i] <- summary(m.b)$concordance['C']
        C.tt[i] <- I(-dxy.cens(predict(m.b, newdata=dd[-sb,]),
                               Surv(dd$T[-sb],
                                    dd$DeathBreast[-sb]))/2 + 0.5)['Dxy']
    }
    boot[m] <- mean(C.tr-C.tt)
}

                                        # Calculate ANOVAs
anovaplus <- lapply(erplus[, models],
                    function(x) anova(piplus,
                                      coxph(survplus ~ offset(PREDICT) + x,
                                            data = erplus)))

# Extract and store model results
summary_plus <- lapply(multiplus, summary)

## Now with bootstrap
boot <- NULL
for (m in models) {
    n <- nrow(erplus)
    C.tr <- numeric(B)
    C.tt <- numeric(B)
    B <- 100
    dd <- erplus[,c('T', 'DeathBreast', 'PREDICT', m)]
    colnames(dd)[4] <- "x"
    for (i in 1:B) {
        sb <- sample(1:n, replace = TRUE)
        m.b <- coxph(Surv(T, DeathBreast) ~ offset(PREDICT) + x,
                     data=dd[sb,])
        C.tr[i] <- summary(m.b)$concordance['C']
        C.tt[i] <- I(-dxy.cens(predict(m.b, newdata=dd[-sb,]),
                               Surv(dd$T[-sb],
                                    dd$DeathBreast[-sb]))/2 + 0.5)['Dxy']
    }
    boot[m] <- mean(C.tr-C.tt)
}


multivar_erplus <- as_tibble(cbind(

  # Populate with info from summaries
  model = paste("PREDICT + ", models),
  coefs = sapply(summary_plus, function(x) x$coefficients[1]),
  loglik = sapply(summary_plus, function(x) x$loglik[2]),
  ses = sapply(summary_plus, function(x) x$coefficients[1,3]),
  pvals = sapply(summary_plus, function(x) x$coefficients[1,5]),
  c_index = sapply(summary_plus, function(x) x$concordance[1]),
  c_ind_se = sapply(summary_plus, function(x) x$concordance[2]),
    c_ind_optim = boot)) %>%


  # Convert to numeric
  mutate(across(.cols = c(2:8), .fns = ~ as.numeric(.))) %>%

  # Append PREDICT separately due to difference in model summary
  add_row(model = "PREDICT",
          coefs = NA_integer_,
          loglik = piplus$loglik,
          ses = NA_integer_,
          pvals = NA_integer_,
          c_index = piplus$concordance[6],
          c_ind_se = piplus$concordance[7],
          c_ind_optim= NA_integer_) %>%

  # Neaten up appearance and calculate CIs
  transmute(model = model,
            coefs = coefs,
            hr = signif(exp(coefs), 3),
            log_likelihood = signif(loglik, 5),
            standard_error = signif(ses,3),
            hr_lci = signif(hr - qnorm(.975) * ses, 3),
            hr_uci = signif(hr + qnorm(.975) * ses, 3),
            p_value = signif(pvals, 3),
            concordance_index = signif(c_index, 3),
            c_index_lci = signif(c_index - qnorm(.975) * c_ind_se, 3),
            c_index_uci = signif(c_index + qnorm(.975) * c_ind_se, 3),
            c_ind_optim=signif(c_ind_optim, 3))

## rm(summary_plus, multiplus, piplus)
rm(summary_plus, piplus)

# Calculation of multivariable models in ER- patients
piminus <- summary(coxph(survminus ~ offset(PREDICT), data = erminus))

multiminus <- lapply(erminus[, models],
                     function(x) coxph(survminus ~ offset(PREDICT) + x,
                                       data = erminus))

# Calculate ANOVAs
anovaminus <- lapply(erminus[, models],
                     function(x) anova(piminus,
                                       coxph(survminus ~ offset(PREDICT) + x,
                                             data = erminus)))

# Extract and store model results
summary_minus <- lapply(multiminus, summary)

## Now with bootstrap
boot <- NULL
for (m in models) {
    n <- nrow(erminus)
    C.tr <- numeric(B)
    C.tt <- numeric(B)
    B <- 100
    dd <- erminus[,c('T', 'DeathBreast', 'PREDICT', m)]
    colnames(dd)[4] <- "x"
    for (i in 1:B) {
        sb <- sample(1:n, replace = TRUE)
        m.b <- coxph(Surv(T, DeathBreast) ~ offset(PREDICT) + x,
                     data=dd[sb,])
        C.tr[i] <- summary(m.b)$concordance['C']
        C.tt[i] <- I(-dxy.cens(predict(m.b, newdata=dd[-sb,]),
                               Surv(dd$T[-sb],
                                    dd$DeathBreast[-sb]))/2 + 0.5)['Dxy']
    }
    boot[m] <- mean(C.tr-C.tt)
}

multivar_erminus <- as_tibble(cbind(

  # Populate with info from summaries
  model = paste("PREDICT +", models),
  coefs = sapply(summary_minus, function(x) x$coefficients[1]),
  loglik = sapply(summary_minus, function(x) x$loglik[2]),
  ses = sapply(summary_minus, function(x) x$coefficients[1,3]),
  pvals = sapply(summary_minus, function(x) x$coefficients[1,5]),
  c_index = sapply(summary_minus, function(x) x$concordance[1]),
  c_ind_se = sapply(summary_minus, function(x) x$concordance[2]),    c_ind_optim = boot)) %>%


  # Convert to numeric
  mutate(across(.cols = c(2:8), .fns = ~ as.numeric(.))) %>%

  # Append PREDICT separately due to difference in model summary
  add_row(model = "PREDICT",
          coefs = NA_integer_,
          loglik = piminus$loglik,
          ses = NA_integer_,
          pvals = NA_integer_,
          c_index = piminus$concordance[6],
          c_ind_se = piminus$concordance[7],
          c_ind_optim = NA_integer_) %>%

  # Neaten up appearance and calculate CIs
  transmute(model = model,
            coefs = coefs,
            hr = signif(exp(coefs), 3),
            log_likelihood = signif(loglik, 5),
            standard_error = signif(ses,3),
            hr_lci = signif(hr - qnorm(.975) * ses, 3),
            hr_uci = signif(hr + qnorm(.975) * ses, 3),
            p_value = signif(pvals, 3),
            concordance_index = signif(c_index, 3),
            c_index_lci = signif(c_index - qnorm(.975) * c_ind_se, 3),
            c_index_uci = signif(c_index + qnorm(.975) * c_ind_se, 3),
            c_ind_optim =signif(c_ind_optim, 3))

## rm(summary_minus, piminus, erplus, erminus, survplus, survminus)
rm(summary_minus, multiminus, piminus, erplus, erminus, survplus, survminus)

# Produce single table of multivariable models and write to .csv
multivar <- bind_rows(multivar_erplus, multivar_erminus, .id = "ER") %>%
  mutate(ER = case_when(ER == 1 ~ "Positive", ER == 2 ~ "Negative")) %>%

  # Tidy for final table
  transmute(
    ER = ER,
    Model = model,
    "HR (95%CI)" = paste(hr, " (", hr_lci, "-", hr_uci, ")", sep = ""),
    "Log-likelihood" = log_likelihood,
    "Model fit p-value" = p_value,
    "c-index (95%CI)" = paste(concordance_index, " (", c_index_lci, "-", c_index_uci, ")", sep = ""),
    "c index optim"=c_ind_optim)

write_csv(multivar, "multivariable_coxph.csv")

rm(univar, univar_erminus, univar_erplus, multivar, anovaplus, anovaminus)

# Modification of PREDICT with new genomic score variable ----

erplus_scalar <- sapply(models, function(x) {

  coef <- multivar_erplus %>%

    # Obtain model coefficients from multivar table
    dplyr::select(model, coefs) %>%
    mutate(model = c(models, "PREDICT")) %>%
    drop_na()  %>%

    # Obtain coefficient for the input model
    dplyr::filter(model == x) %>%
    dplyr::select(coefs) %>%
    as.numeric(.)

  # Pull correct model column from outcomes
  x <- outcomes %>%
    ungroup() %>%
    dplyr::select(x) %>%

    #Calculate scalar
    summarise(scalar = -log(mean(exp(coef * .data[[x]]),na.rm = TRUE))/coef)

  as.numeric(x)
})

erminus_scalar <- sapply(models, function(x) {

  coef <- multivar_erminus %>%

    # Obtain model coefficients from multivar table
    dplyr::select(model, coefs) %>%
    mutate(model = c(models, "PREDICT")) %>%
    drop_na()  %>%

    # Obtain coefficient for the input model
    dplyr::filter(model == x) %>%
    dplyr::select(coefs) %>%
    as.numeric(.)

  # Pull correct model column from outcomes
  x <- outcomes %>%
    ungroup() %>%
    dplyr::select(x) %>%

    #Calculate scalar
    summarise(scalar = -log(mean(exp(coef * .data[[x]]),na.rm = TRUE))/coef)

  as.numeric(x)
})

# Check the new mean == 1
outcomes %>%
  mutate(new_ODx = case_when(ER.Expr == 1 ~ ODx +
                               as.numeric(erplus_scalar["ODx"]),
                             ER.Expr == 0 ~ ODx +
                               as.numeric(erminus_scalar["ODx"]))) %>%
  ungroup() %>%
  transmute(HR = case_when(ER.Expr == 1 ~ mean(exp(0.005201091 * new_ODx),
                                               na.rm = TRUE),
                           ER.Expr == 0 ~ mean(exp(-0.003081556 * new_ODx),
                                               na.rm = TRUE))) %>%
  summarise(HR = mean(HR))

# Create modified PREDICT function
mod_predict_fn <- function (data = outcomes, year = 10, age.start = Dx_Age,
                            size = Size, grade = Grade, nodes = Nodes,
                            er = ER.Expr, her2 = Her2.Expr, ki67 = Ki67,
                            generation = 2, horm = HT, genome) {
  age.start <- deparse(substitute(age.start))
  size <- deparse(substitute(size))
  grade <- deparse(substitute(grade))
  nodes <- deparse(substitute(nodes))
  er <- deparse(substitute(er))
  her2 <- deparse(substitute(her2))
  ki67 <- deparse(substitute(ki67))
  horm <- deparse(substitute(horm))
  genome <- deparse(substitute(genome))

  predict2.1s <- function(age.start, size, grade, nodes,
                          er, her2, ki67, genome, generation = 2, horm,
                          traz = 0, bis = 0, score) {
    screen <- 0.204
    grade <- ifelse(grade == 9, 2.13, grade)
    time <- c(1:15)
    age <- age.start - 1 + time
    grade.val <- ifelse(er == 1, grade, ifelse(grade == 2 || grade == 3, 1, 0))
    age.mfp.1 <- ifelse(er == 1, (age.start/10)^-2 - 0.0287449295,
                        age.start - 56.3254902)
    age.beta.1 <- ifelse(er == 1, 34.53642, 0.0089827)
    age.mfp.2 <- ifelse(er == 1, (age.start/10)^-2 * log(age.start/10) -
                          0.0510121013, 0)
    age.beta.2 <- ifelse(er == 1, -34.20342, 0)
    size.mfp <- ifelse(er == 1, log(size/100) + 1.545233938,
                       (size/100)^0.5 - 0.5090456276)
    size.beta <- ifelse(er == 1, 0.7530729, 2.093446)
    nodes.mfp <- ifelse(er == 1, log((nodes + 1)/10) + 1.387566896,
                        log((nodes + 1)/10) + 1.086916249)
    nodes.beta <- ifelse(er == 1, 0.7060723, 0.6260541)
    grade.beta <- ifelse(er == 1, 0.746655, 1.129091)
    screen.beta <- 0
    her2.beta <- ifelse(her2 == 1, 0.2413, ifelse(her2 == 0, -0.0762, 0))
    ki67.beta <- ifelse(ki67 == 1 & er == 1, 0.14904,
                        ifelse(ki67 == 0 & er == 1, -0.11333, 0))

    genome.beta <- ifelse(er == 1,
                          as.numeric(multivar_erplus[str_detect(
                            multivar_erplus$model,
                            score), "coefs"][1,1]),
                          as.numeric(multivar_erminus[str_detect(
                            multivar_erminus$model,
                            score), "coefs"][1,1]))

    genome.mfp <- ifelse(er == 1,
                         genome + erplus_scalar[as.character(score)],
                         genome + erminus_scalar[as.character(score)])
    r.prop <- 0.69
    r.breast <- 0.82
    r.other <- 1.07
    mi <- 0.0698252 * ((age.start/10)^2 - 34.23391957)
    pi <- age.beta.1 * age.mfp.1 + age.beta.2 * age.mfp.2 +
      size.beta * size.mfp + nodes.beta * nodes.mfp + grade.beta *
      grade.val + screen.beta * screen + her2.beta + ki67.beta +
      genome.beta * genome.mfp

    c <- -0.248 # All chemo assessed as if second generation
    h <- ifelse(horm == 1 & er == 1, -0.3857, 0)
    t <- 0
    b <- 0
    hc <- h + c
    ht <- h + t
    hb <- h + b
    ct <- c + t
    cb <- c + b
    tb <- t + b
    hct <- h + c + t
    hcb <- h + c + b
    htb <- h + t + b
    ctb <- c + t + b
    hctb <- h + c + t + b
    base.m.cum.oth <- exp(-6.052919 + (1.079863 * log(time)) +
                            (0.3255321 * time^0.5))
    s.cum.oth <- exp(-exp(mi) * base.m.cum.oth)
    base.m.oth <- base.m.cum.oth
    for (i in 2:15) {
      base.m.oth[i] <- base.m.cum.oth[i] - base.m.cum.oth[i -
                                                            1]
    }
    rx.oth <- c(surg = 0, h = 0, c = 0, t = 0, b = 0, hc = 0,
                ht = 0, hb = 0, ct = 0, cb = 0, tb = 0, hct = 0,
                hcb = 0, htb = 0, ctb = 0, hctb = 0)
    cols <- length(rx.oth)
    m.oth.rx <- sapply(rx.oth, function(rx.oth, x.vector = base.m.oth) {
      output <- x.vector * exp(mi + rx.oth)
      return(output)
    })
    m.cum.oth.rx <- apply(m.oth.rx, 2, cumsum)
    s.cum.oth.rx <- exp(-m.cum.oth.rx)
    m.cum.oth.rx <- 1 - s.cum.oth.rx
    m.oth.rx <- m.cum.oth.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.oth.rx[i, j] <- m.cum.oth.rx[i, j] - m.cum.oth.rx[i -
                                                              1, j]
      }
    }
    if (er == 1) {
      base.m.cum.br <- exp(0.7424402 - 7.527762/time^0.5 -
                             1.812513 * log(time)/time^0.5)
    }
    else {
      base.m.cum.br <- exp(-1.156036 + 0.4707332/time^2 -
                             3.51355/time)
    }
    base.m.br <- base.m.cum.br
    for (i in 2:15) {
      base.m.br[i] <- base.m.cum.br[i] - base.m.cum.br[i -
                                                         1]
    }
    rx <- c(surg = 0, h = h, c = c, t = t, b = b, hc = hc,
            ht = ht, hb = hb, ct = ct, cb = cb, tb = tb, hct = hct,
            hcb = hcb, htb = htb, ctb = ctb, hctb = hctb)
    m.br.rx <- sapply(rx, function(rx, x.vector = base.m.br) {
      output <- x.vector * exp(pi + rx)
      return(output)
    })
    m.cum.br.rx <- apply(m.br.rx, 2, cumsum)
    s.cum.br.rx <- exp(-m.cum.br.rx)
    m.cum.br.rx <- 1 - s.cum.br.rx
    m.br.rx <- m.cum.br.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.br.rx[i, j] <- m.cum.br.rx[i, j] - m.cum.br.rx[i -
                                                           1, j]
      }
    }
    m.cum.all.rx <- 1 - s.cum.oth.rx * s.cum.br.rx
    s.cum.all.rx <- 100 - 100 * m.cum.all.rx
    m.all.rx <- m.cum.all.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.all.rx[i, j] <- m.cum.all.rx[i, j] - m.cum.all.rx[i -
                                                              1, j]
      }
    }
    prop.br.rx <- m.br.rx/(m.br.rx + m.oth.rx)
    pred.m.br.rx <- prop.br.rx * m.all.rx
    pred.cum.br.rx <- apply(pred.m.br.rx, 2, cumsum)
    pred.m.oth.rx <- m.all.rx - pred.m.br.rx
    pred.cum.oth.rx <- apply(pred.m.oth.rx, 2, cumsum)
    pred.cum.all.rx <- pred.cum.br.rx + pred.cum.oth.rx
    benefits2.1 <- 100 * (pred.cum.all.rx[, 1] - pred.cum.all.rx)
    Overall <- round((1 - pred.cum.all.rx[year, 1]) * 100,
                     2)
    horm.ben <- round(benefits2.1[year, 2], 2)
    chemo.ben <- round(benefits2.1[year, 6] - benefits2.1[year,
                                                          2], 2)
    return(c(pi, Overall, chemo.ben))
  }
  {
    N <- nrow(data)
    results <- array(c(NA, NA), c(N, 3))
    P.I <- paste("PREDICT", genome, "pi", sep = "_")
    O.S <- paste("PREDICT", genome, "OS", sep = "_")
    C.B <- paste("PREDICT", genome, "deltaCT", sep = "_")
    colnames(results) <- c(P.I, O.S, C.B)
  }
  for (n in 1:N) {
    results[n, ] <- predict2.1s(age.start = data[[age.start]][n],
                                size = data[[size]][n],
                                grade = data[[grade]][n],
                                nodes = data[[nodes]][n],
                                er = data[[er]][n],
                                her2 = data[[her2]][n],
                                ki67 = data[[ki67]][n],
                                score = genome,
                                genome = data[[genome]][n],
                                horm = data[[horm]][n])
  }
  cbind.data.frame("METABRIC.ID" = outcomes$METABRIC.ID, results)
}

outcomes <- outcomes %>%

  # Call modified PREDICT function for all continuous scores
  full_join(., as_tibble(mod_predict_fn(genome = ODx)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = EP)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = MP)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = ROR)),
            by = "METABRIC.ID")

rm(erminus_scalar, erplus_scalar, mod_predict_fn, multivar_erplus, multivar_erminus)

# Compare calibration----

#### By ER status
# Observed survival at 10y in ER+
erplus <- filter(outcomes, ER.Expr == 1, !is.na(PREDICT))
survplus <- with(erplus, Surv(`T`, DeathBreast))

km_erplus <- survfit(survplus ~ 1, data = erplus)
obs_surv_erplus <- km_erplus$surv[which(km_erplus$time == 3652)] * 100
lci_erplus <- km_erplus$lower[which(km_erplus$time == 3652)] * 100
uci_erplus <- km_erplus$upper[which(km_erplus$time == 3652)] * 100

# Observed survival at 10y in ER-
erminus <- filter(outcomes, ER.Expr == 0, !is.na(PREDICT))
survminus <-  with(erminus, Surv(`T`, DeathBreast))

km_erminus <- survfit(survminus ~ 1, data = erminus)
obs_surv_erminus <- km_erminus$surv[which(km_erminus$time > 3652)][1] * 100
lci_erminus <- km_erminus$lower[which(km_erminus$time > 3652)][1] * 100
uci_erminus <- km_erminus$upper[which(km_erminus$time > 3652)][1] * 100

rm(km_erplus, km_erminus)

# Predicted survival based on modified PREDICT models
pred_erplus <- outcomes %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

pred_erminus <- outcomes %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# Build table of absolute difference in calibration by ER status
calibration_er <- bind_rows(pred_erplus, pred_erminus) %>%
  add_column(METABRIC = c(obs_surv_erplus, obs_surv_erminus)) %>%
  t() %>%
  as_tibble() %>%
  rename(ERplus = V1, ERminus = V2) %>%
  mutate(ERplus_delta = ERplus - obs_surv_erplus,
         ERminus_delta = ERminus - obs_surv_erminus) %>%
  mutate(across(everything(), ~ round(.x, digits = 1))) %>%
  dplyr::select(order(colnames(.))) %>%
  add_column(Model = c("PREDICT", models[c(-6,-7)], "METABRIC"),
             .before = TRUE)

newnames <- colnames(calibration_er) %>%
  as_tibble() %>%
  mutate(value = case_when(value == "ERminus" ~ paste(value, " (n = ", nrow(erminus), ")", sep = ""),
                           value == "ERplus" ~ paste(value, " (n = ", nrow(erplus), ")", sep = ""),
                           TRUE ~ value))

colnames(calibration_er) <- as.vector(newnames$value)

write_csv(calibration_er, "calibration_erstatus.csv")

## Bootstrap procedure for calibration
models <- c("ODx", "EP", "MP", "ROR")
set.seed(43534)
B <- 100
    opt.pos <- matrix(NA, 6, B)
    opt.neg <- matrix(NA, 6, B)
    n1 <- sum(outcomes$ER.Expr==1)
    n2 <- sum(outcomes$ER.Expr==0)
    for (i in 1:B) {
        sb1 <- sample(1:n1, replace = TRUE)
        sb2 <- sample(1:n2, replace = TRUE)
        outcomes.b <- rbind(subset(outcomes, ER.Expr==1)[sb1,1:40],
                            subset(outcomes, ER.Expr==0)[sb2,1:40])
        erplus <- subset(outcomes.b, ER.Expr == 1)[,c('T', 'DeathBreast', 'PREDICT','ODx', 'EP','MP','ROR')]
        erminus <- subset(outcomes.b, ER.Expr == 0)[,c('T', 'DeathBreast', 'PREDICT','ODx', 'EP','MP','ROR')]

        piplus <- summary(coxph(Surv(T, DeathBreast) ~ offset(PREDICT), data = erplus))

        multiplus <- lapply(erplus[, models],
                            function(x) coxph(Surv(T, DeathBreast) ~ offset(PREDICT) + x,
                                              data = erplus))
        summary_plus <- lapply(multiplus, summary)

        multivar_erplus <- as_tibble(cbind(
                                        # Populate with info from summaries
  model = paste("PREDICT + ", models),
  coefs = sapply(summary_plus, function(x) x$coefficients[1]),
  loglik = sapply(summary_plus, function(x) x$loglik[2]),
  ses = sapply(summary_plus, function(x) x$coefficients[1,3]),
  pvals = sapply(summary_plus, function(x) x$coefficients[1,5]),
  c_index = sapply(summary_plus, function(x) x$concordance[1]),
  c_ind_se = sapply(summary_plus, function(x) x$concordance[2]))) %>%


  # Convert to numeric
  mutate(across(.cols = c(2:7), .fns = ~ as.numeric(.))) %>%

  # Append PREDICT separately due to difference in model summary
  add_row(model = "PREDICT",
          coefs = NA_integer_,
          loglik = piplus$loglik,
          ses = NA_integer_,
          pvals = NA_integer_,
          c_index = piplus$concordance[6],
          c_ind_se = piplus$concordance[7]) %>%


  # Neaten up appearance and calculate CIs
  transmute(model = model,
            coefs = coefs,
            hr = signif(exp(coefs), 3),
            log_likelihood = signif(loglik, 5),
            standard_error = signif(ses,3),
            hr_lci = signif(hr - qnorm(.975) * ses, 3),
            hr_uci = signif(hr + qnorm(.975) * ses, 3),
            p_value = signif(pvals, 3),
            concordance_index = signif(c_index, 3),
            c_index_lci = signif(c_index - qnorm(.975) * c_ind_se, 3),
            c_index_uci = signif(c_index + qnorm(.975) * c_ind_se, 3))


        erplus_scalar <- sapply(models, function(x) {

            coef <- multivar_erplus %>%

    # Obtain model coefficients from multivar table
    dplyr::select(model, coefs) %>%
    mutate(model = c(models, "PREDICT")) %>%
    drop_na()  %>%

    # Obtain coefficient for the input model
    dplyr::filter(model == x) %>%
    dplyr::select(coefs) %>%
    as.numeric(.)

  # Pull correct model column from outcomes
  x <- outcomes.b %>%
    ungroup() %>%
    dplyr::select(x) %>%

    #Calculate scalar
    summarise(scalar = -log(mean(exp(coef * .data[[x]]),na.rm = TRUE))/coef)

  as.numeric(x)
})

        piminus <- summary(coxph(Surv(T, DeathBreast) ~ offset(PREDICT), data = erminus))

        multiminus <- lapply(erminus[, models],
                            function(x) coxph(Surv(T, DeathBreast) ~ offset(PREDICT) + x,
                                              data = erminus))
        summary_minus <- lapply(multiminus, summary)

        multivar_erminus <- as_tibble(cbind(
                                        # Populate with info from summaries
  model = paste("PREDICT + ", models),
  coefs = sapply(summary_minus, function(x) x$coefficients[1]),
  loglik = sapply(summary_minus, function(x) x$loglik[2]),
  ses = sapply(summary_minus, function(x) x$coefficients[1,3]),
  pvals = sapply(summary_minus, function(x) x$coefficients[1,5]),
  c_index = sapply(summary_minus, function(x) x$concordance[1]),
  c_ind_se = sapply(summary_minus, function(x) x$concordance[2]))) %>%


  # Convert to numeric
  mutate(across(.cols = c(2:7), .fns = ~ as.numeric(.))) %>%

  # Append PREDICT separately due to difference in model summary
  add_row(model = "PREDICT",
          coefs = NA_integer_,
          loglik = piminus$loglik,
          ses = NA_integer_,
          pvals = NA_integer_,
          c_index = piminus$concordance[6],
          c_ind_se = piminus$concordance[7]) %>%


  # Neaten up appearance and calculate CIs
  transmute(model = model,
            coefs = coefs,
            hr = signif(exp(coefs), 3),
            log_likelihood = signif(loglik, 5),
            standard_error = signif(ses,3),
            hr_lci = signif(hr - qnorm(.975) * ses, 3),
            hr_uci = signif(hr + qnorm(.975) * ses, 3),
            p_value = signif(pvals, 3),
            concordance_index = signif(c_index, 3),
            c_index_lci = signif(c_index - qnorm(.975) * c_ind_se, 3),
            c_index_uci = signif(c_index + qnorm(.975) * c_ind_se, 3))



        erminus_scalar <- sapply(models, function(x) {

  coef <- multivar_erminus %>%

    # Obtain model coefficients from multivar table
    dplyr::select(model, coefs) %>%
    mutate(model = c(models, "PREDICT")) %>%
    drop_na()  %>%

    # Obtain coefficient for the input model
    dplyr::filter(model == x) %>%
    dplyr::select(coefs) %>%
    as.numeric(.)

  # Pull correct model column from outcomes
  x <- outcomes.b %>%
    ungroup() %>%
    dplyr::select(x) %>%

    #Calculate scalar
    summarise(scalar = -log(mean(exp(coef * .data[[x]]),na.rm = TRUE))/coef)

  as.numeric(x)
})

# Check the new mean == 1
outcomes.b %>%
  mutate(new_ODx = case_when(ER.Expr == 1 ~ ODx +
                               as.numeric(erplus_scalar["ODx"]),
                             ER.Expr == 0 ~ ODx +
                               as.numeric(erminus_scalar["ODx"]))) %>%
  ungroup() %>%
  transmute(HR = case_when(ER.Expr == 1 ~ mean(exp(0.005201091 * new_ODx),
                                               na.rm = TRUE),
                           ER.Expr == 0 ~ mean(exp(-0.003081556 * new_ODx),
                                               na.rm = TRUE))) %>%
  summarise(HR = mean(HR))

# Modification of PREDICT with new genomic score variable ----


mod_predict_fn <- function (data = outcomes.b, year = 10, age.start = Dx_Age,
                            size = Size, grade = Grade, nodes = Nodes,
                            er = ER.Expr, her2 = Her2.Expr, ki67 = Ki67,
                            generation = 2, horm = HT, genome) {
  age.start <- deparse(substitute(age.start))
  size <- deparse(substitute(size))
  grade <- deparse(substitute(grade))
  nodes <- deparse(substitute(nodes))
  er <- deparse(substitute(er))
  her2 <- deparse(substitute(her2))
  ki67 <- deparse(substitute(ki67))
  horm <- deparse(substitute(horm))
  genome <- deparse(substitute(genome))

  predict2.1s <- function(age.start, size, grade, nodes,
                          er, her2, ki67, genome, generation = 2, horm,
                          traz = 0, bis = 0, score, model.erplus, model.erminus) {
    screen <- 0.204
    grade <- ifelse(grade == 9, 2.13, grade)
    time <- c(1:15)
    age <- age.start - 1 + time
    grade.val <- ifelse(er == 1, grade, ifelse(grade == 2 || grade == 3, 1, 0))
    age.mfp.1 <- ifelse(er == 1, (age.start/10)^-2 - 0.0287449295,
                        age.start - 56.3254902)
    age.beta.1 <- ifelse(er == 1, 34.53642, 0.0089827)
    age.mfp.2 <- ifelse(er == 1, (age.start/10)^-2 * log(age.start/10) -
                          0.0510121013, 0)
    age.beta.2 <- ifelse(er == 1, -34.20342, 0)
    size.mfp <- ifelse(er == 1, log(size/100) + 1.545233938,
                       (size/100)^0.5 - 0.5090456276)
    size.beta <- ifelse(er == 1, 0.7530729, 2.093446)
    nodes.mfp <- ifelse(er == 1, log((nodes + 1)/10) + 1.387566896,
                        log((nodes + 1)/10) + 1.086916249)
    nodes.beta <- ifelse(er == 1, 0.7060723, 0.6260541)
    grade.beta <- ifelse(er == 1, 0.746655, 1.129091)
    screen.beta <- 0
    her2.beta <- ifelse(her2 == 1, 0.2413, ifelse(her2 == 0, -0.0762, 0))
    ki67.beta <- ifelse(ki67 == 1 & er == 1, 0.14904,
                        ifelse(ki67 == 0 & er == 1, -0.11333, 0))

    genome.beta <- ifelse(er == 1,
                          coef(multiplus[[score]]),
                          coef(multiminus[[score]]))

    genome.mfp <- ifelse(er == 1,
                         genome + erplus_scalar[as.character(score)],
                         genome + erminus_scalar[as.character(score)])
    r.prop <- 0.69
    r.breast <- 0.82
    r.other <- 1.07
    mi <- 0.0698252 * ((age.start/10)^2 - 34.23391957)
    pi <- age.beta.1 * age.mfp.1 + age.beta.2 * age.mfp.2 +
      size.beta * size.mfp + nodes.beta * nodes.mfp + grade.beta *
      grade.val + screen.beta * screen + her2.beta + ki67.beta +
      genome.beta * genome.mfp

    c <- -0.248 # All chemo assessed as if second generation
    h <- ifelse(horm == 1 & er == 1, -0.3857, 0)
    t <- 0
    b <- 0
    hc <- h + c
    ht <- h + t
    hb <- h + b
    ct <- c + t
    cb <- c + b
    tb <- t + b
    hct <- h + c + t
    hcb <- h + c + b
    htb <- h + t + b
    ctb <- c + t + b
    hctb <- h + c + t + b
    base.m.cum.oth <- exp(-6.052919 + (1.079863 * log(time)) +
                            (0.3255321 * time^0.5))
    s.cum.oth <- exp(-exp(mi) * base.m.cum.oth)
    base.m.oth <- base.m.cum.oth
    for (i in 2:15) {
      base.m.oth[i] <- base.m.cum.oth[i] - base.m.cum.oth[i -
                                                            1]
    }
    rx.oth <- c(surg = 0, h = 0, c = 0, t = 0, b = 0, hc = 0,
                ht = 0, hb = 0, ct = 0, cb = 0, tb = 0, hct = 0,
                hcb = 0, htb = 0, ctb = 0, hctb = 0)
    cols <- length(rx.oth)
    m.oth.rx <- sapply(rx.oth, function(rx.oth, x.vector = base.m.oth) {
      output <- x.vector * exp(mi + rx.oth)
      return(output)
    })
    m.cum.oth.rx <- apply(m.oth.rx, 2, cumsum)
    s.cum.oth.rx <- exp(-m.cum.oth.rx)
    m.cum.oth.rx <- 1 - s.cum.oth.rx
    m.oth.rx <- m.cum.oth.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.oth.rx[i, j] <- m.cum.oth.rx[i, j] - m.cum.oth.rx[i -
                                                              1, j]
      }
    }
    if (er == 1) {
      base.m.cum.br <- exp(0.7424402 - 7.527762/time^0.5 -
                             1.812513 * log(time)/time^0.5)
    }
    else {
      base.m.cum.br <- exp(-1.156036 + 0.4707332/time^2 -
                             3.51355/time)
    }
    base.m.br <- base.m.cum.br
    for (i in 2:15) {
      base.m.br[i] <- base.m.cum.br[i] - base.m.cum.br[i -
                                                         1]
    }
    rx <- c(surg = 0, h = h, c = c, t = t, b = b, hc = hc,
            ht = ht, hb = hb, ct = ct, cb = cb, tb = tb, hct = hct,
            hcb = hcb, htb = htb, ctb = ctb, hctb = hctb)
    m.br.rx <- sapply(rx, function(rx, x.vector = base.m.br) {
      output <- x.vector * exp(pi + rx)
      return(output)
    })
    m.cum.br.rx <- apply(m.br.rx, 2, cumsum)
    s.cum.br.rx <- exp(-m.cum.br.rx)
    m.cum.br.rx <- 1 - s.cum.br.rx
    m.br.rx <- m.cum.br.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.br.rx[i, j] <- m.cum.br.rx[i, j] - m.cum.br.rx[i -
                                                           1, j]
      }
    }
    m.cum.all.rx <- 1 - s.cum.oth.rx * s.cum.br.rx
    s.cum.all.rx <- 100 - 100 * m.cum.all.rx
    m.all.rx <- m.cum.all.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.all.rx[i, j] <- m.cum.all.rx[i, j] - m.cum.all.rx[i -
                                                              1, j]
      }
    }
    prop.br.rx <- m.br.rx/(m.br.rx + m.oth.rx)
    pred.m.br.rx <- prop.br.rx * m.all.rx
    pred.cum.br.rx <- apply(pred.m.br.rx, 2, cumsum)
    pred.m.oth.rx <- m.all.rx - pred.m.br.rx
    pred.cum.oth.rx <- apply(pred.m.oth.rx, 2, cumsum)
    pred.cum.all.rx <- pred.cum.br.rx + pred.cum.oth.rx
    benefits2.1 <- 100 * (pred.cum.all.rx[, 1] - pred.cum.all.rx)
    Overall <- round((1 - pred.cum.all.rx[year, 1]) * 100,
                     2)
    horm.ben <- round(benefits2.1[year, 2], 2)
    chemo.ben <- round(benefits2.1[year, 6] - benefits2.1[year,
                                                          2], 2)
    return(c(pi, Overall, chemo.ben))
  }
  {
    N <- nrow(data)
    results <- array(c(NA, NA), c(N, 3))
    P.I <- paste("PREDICT", genome, "pi", sep = "_")
    O.S <- paste("PREDICT", genome, "OS", sep = "_")
    C.B <- paste("PREDICT", genome, "deltaCT", sep = "_")
    colnames(results) <- c(P.I, O.S, C.B)
  }
  for (n in 1:N) {
    results[n, ] <- predict2.1s(age.start = data[[age.start]][n],
                                size = data[[size]][n],
                                grade = data[[grade]][n],
                                nodes = data[[nodes]][n],
                                er = data[[er]][n],
                                her2 = data[[her2]][n],
                                ki67 = data[[ki67]][n],
                                score = genome,
                                genome = data[[genome]][n],
                                horm = data[[horm]][n])
  }
  cbind.data.frame("METABRIC.ID" = outcomes.b$METABRIC.ID, results)
}

        outcomes.b <- outcomes.b %>%

  # Call modified PREDICT function for all continuous scores
  full_join(., as_tibble(mod_predict_fn(genome = ODx)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = EP)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = MP)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = ROR)),
            by = "METABRIC.ID")

erplus <- filter(outcomes.b, ER.Expr == 1, !is.na(PREDICT))
survplus <- with(erplus, Surv(`T`, DeathBreast))

km_erplus <- survfit(survplus ~ 1, data = erplus)
obs_surv_erplus <- summary(km_erplus, times=3652)$surv * 100

# Observed survival at 10y in ER-
erminus <- filter(outcomes.b, ER.Expr == 0, !is.na(PREDICT))
survminus <-  with(erminus, Surv(`T`, DeathBreast))

km_erminus <- survfit(survminus ~ 1, data = erminus)
obs_surv_erminus <- summary(km_erminus, times=3652)$surv * 100

        pred_erplus <- outcomes.b %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

pred_erminus <- outcomes.b %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# Build table of absolute difference in calibration by ER status
calibration_er <- bind_rows(pred_erplus, pred_erminus) %>%
  add_column(METABRIC = c(obs_surv_erplus, obs_surv_erminus)) %>%
  t() %>%
  as_tibble() %>%
  rename(ERplus = V1, ERminus = V2) %>%
  mutate(ERplus_delta = ERplus - obs_surv_erplus,
         ERminus_delta = ERminus - obs_surv_erminus) %>%
  mutate(across(everything(), ~ round(.x, digits = 1))) %>%
  dplyr::select(order(colnames(.))) %>%
  add_column(Model = c("PREDICT", models[c(-6,-7)], "METABRIC"),
             .before = TRUE)

newnames <- colnames(calibration_er) %>%

    as_tibble() %>%
  mutate(value = case_when(value == "ERminus" ~ paste(value, " (n = ", nrow(erminus), ")", sep = ""),
                           value == "ERplus" ~ paste(value, " (n = ", nrow(erplus), ")", sep = ""),
                           TRUE ~ value))

colnames(calibration_er) <- as.vector(newnames$value)


        ## Now with the test dataset

        outcomes.t <- rbind(subset(outcomes, ER.Expr==1)[-sb1,1:40],
                            subset(outcomes, ER.Expr==0)[-sb2,1:40])
        erplus <- subset(outcomes.t, ER.Expr == 1)[,c('T', 'DeathBreast', 'PREDICT','ODx', 'EP','MP','ROR')]
        erminus <- subset(outcomes.t, ER.Expr == 0)[,c('T', 'DeathBreast', 'PREDICT','ODx', 'EP','MP','ROR')]

        mod_predict_fn <- function (data = outcomes.t, year = 10, age.start = Dx_Age,
                            size = Size, grade = Grade, nodes = Nodes,
                            er = ER.Expr, her2 = Her2.Expr, ki67 = Ki67,
                            generation = 2, horm = HT, genome) {
  age.start <- deparse(substitute(age.start))
  size <- deparse(substitute(size))
  grade <- deparse(substitute(grade))
  nodes <- deparse(substitute(nodes))
  er <- deparse(substitute(er))
  her2 <- deparse(substitute(her2))
  ki67 <- deparse(substitute(ki67))
  horm <- deparse(substitute(horm))
  genome <- deparse(substitute(genome))

  predict2.1s <- function(age.start, size, grade, nodes,
                          er, her2, ki67, genome, generation = 2, horm,
                          traz = 0, bis = 0, score, model.erplus, model.erminus) {
    screen <- 0.204
    grade <- ifelse(grade == 9, 2.13, grade)
    time <- c(1:15)
    age <- age.start - 1 + time
    grade.val <- ifelse(er == 1, grade, ifelse(grade == 2 || grade == 3, 1, 0))
    age.mfp.1 <- ifelse(er == 1, (age.start/10)^-2 - 0.0287449295,
                        age.start - 56.3254902)
    age.beta.1 <- ifelse(er == 1, 34.53642, 0.0089827)
    age.mfp.2 <- ifelse(er == 1, (age.start/10)^-2 * log(age.start/10) -
                          0.0510121013, 0)
    age.beta.2 <- ifelse(er == 1, -34.20342, 0)
    size.mfp <- ifelse(er == 1, log(size/100) + 1.545233938,
                       (size/100)^0.5 - 0.5090456276)
    size.beta <- ifelse(er == 1, 0.7530729, 2.093446)
    nodes.mfp <- ifelse(er == 1, log((nodes + 1)/10) + 1.387566896,
                        log((nodes + 1)/10) + 1.086916249)
    nodes.beta <- ifelse(er == 1, 0.7060723, 0.6260541)
    grade.beta <- ifelse(er == 1, 0.746655, 1.129091)
    screen.beta <- 0
    her2.beta <- ifelse(her2 == 1, 0.2413, ifelse(her2 == 0, -0.0762, 0))
    ki67.beta <- ifelse(ki67 == 1 & er == 1, 0.14904,
                        ifelse(ki67 == 0 & er == 1, -0.11333, 0))

    genome.beta <- ifelse(er == 1,
                          coef(multiplus[[score]]),
                          coef(multiminus[[score]]))

    genome.mfp <- ifelse(er == 1,
                         genome + erplus_scalar[as.character(score)],
                         genome + erminus_scalar[as.character(score)])
    r.prop <- 0.69
    r.breast <- 0.82
    r.other <- 1.07
    mi <- 0.0698252 * ((age.start/10)^2 - 34.23391957)
    pi <- age.beta.1 * age.mfp.1 + age.beta.2 * age.mfp.2 +
      size.beta * size.mfp + nodes.beta * nodes.mfp + grade.beta *
      grade.val + screen.beta * screen + her2.beta + ki67.beta +
      genome.beta * genome.mfp

    c <- -0.248 # All chemo assessed as if second generation
    h <- ifelse(horm == 1 & er == 1, -0.3857, 0)
    t <- 0
    b <- 0
    hc <- h + c
    ht <- h + t
    hb <- h + b
    ct <- c + t
    cb <- c + b
    tb <- t + b
    hct <- h + c + t
    hcb <- h + c + b
    htb <- h + t + b
    ctb <- c + t + b
    hctb <- h + c + t + b
    base.m.cum.oth <- exp(-6.052919 + (1.079863 * log(time)) +
                            (0.3255321 * time^0.5))
    s.cum.oth <- exp(-exp(mi) * base.m.cum.oth)
    base.m.oth <- base.m.cum.oth
    for (i in 2:15) {
      base.m.oth[i] <- base.m.cum.oth[i] - base.m.cum.oth[i -
                                                            1]
    }
    rx.oth <- c(surg = 0, h = 0, c = 0, t = 0, b = 0, hc = 0,
                ht = 0, hb = 0, ct = 0, cb = 0, tb = 0, hct = 0,
                hcb = 0, htb = 0, ctb = 0, hctb = 0)
    cols <- length(rx.oth)
    m.oth.rx <- sapply(rx.oth, function(rx.oth, x.vector = base.m.oth) {
      output <- x.vector * exp(mi + rx.oth)
      return(output)
    })
    m.cum.oth.rx <- apply(m.oth.rx, 2, cumsum)
    s.cum.oth.rx <- exp(-m.cum.oth.rx)
    m.cum.oth.rx <- 1 - s.cum.oth.rx
    m.oth.rx <- m.cum.oth.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.oth.rx[i, j] <- m.cum.oth.rx[i, j] - m.cum.oth.rx[i -
                                                              1, j]
      }
    }
    if (er == 1) {
      base.m.cum.br <- exp(0.7424402 - 7.527762/time^0.5 -
                             1.812513 * log(time)/time^0.5)
    }
    else {
      base.m.cum.br <- exp(-1.156036 + 0.4707332/time^2 -
                             3.51355/time)
    }
    base.m.br <- base.m.cum.br
    for (i in 2:15) {
      base.m.br[i] <- base.m.cum.br[i] - base.m.cum.br[i -
                                                         1]
    }
    rx <- c(surg = 0, h = h, c = c, t = t, b = b, hc = hc,
            ht = ht, hb = hb, ct = ct, cb = cb, tb = tb, hct = hct,
            hcb = hcb, htb = htb, ctb = ctb, hctb = hctb)
    m.br.rx <- sapply(rx, function(rx, x.vector = base.m.br) {
      output <- x.vector * exp(pi + rx)
      return(output)
    })
    m.cum.br.rx <- apply(m.br.rx, 2, cumsum)
    s.cum.br.rx <- exp(-m.cum.br.rx)
    m.cum.br.rx <- 1 - s.cum.br.rx
    m.br.rx <- m.cum.br.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.br.rx[i, j] <- m.cum.br.rx[i, j] - m.cum.br.rx[i -
                                                           1, j]
      }
    }
    m.cum.all.rx <- 1 - s.cum.oth.rx * s.cum.br.rx
    s.cum.all.rx <- 100 - 100 * m.cum.all.rx
    m.all.rx <- m.cum.all.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.all.rx[i, j] <- m.cum.all.rx[i, j] - m.cum.all.rx[i -
                                                              1, j]
      }
    }
    prop.br.rx <- m.br.rx/(m.br.rx + m.oth.rx)
    pred.m.br.rx <- prop.br.rx * m.all.rx
    pred.cum.br.rx <- apply(pred.m.br.rx, 2, cumsum)
    pred.m.oth.rx <- m.all.rx - pred.m.br.rx
    pred.cum.oth.rx <- apply(pred.m.oth.rx, 2, cumsum)
    pred.cum.all.rx <- pred.cum.br.rx + pred.cum.oth.rx
    benefits2.1 <- 100 * (pred.cum.all.rx[, 1] - pred.cum.all.rx)
    Overall <- round((1 - pred.cum.all.rx[year, 1]) * 100,
                     2)
    horm.ben <- round(benefits2.1[year, 2], 2)
    chemo.ben <- round(benefits2.1[year, 6] - benefits2.1[year,
                                                          2], 2)
    return(c(pi, Overall, chemo.ben))
  }
  {
    N <- nrow(data)
    results <- array(c(NA, NA), c(N, 3))
    P.I <- paste("PREDICT", genome, "pi", sep = "_")
    O.S <- paste("PREDICT", genome, "OS", sep = "_")
    C.B <- paste("PREDICT", genome, "deltaCT", sep = "_")
    colnames(results) <- c(P.I, O.S, C.B)
  }
  for (n in 1:N) {
    results[n, ] <- predict2.1s(age.start = data[[age.start]][n],
                                size = data[[size]][n],
                                grade = data[[grade]][n],
                                nodes = data[[nodes]][n],
                                er = data[[er]][n],
                                her2 = data[[her2]][n],
                                ki67 = data[[ki67]][n],
                                score = genome,
                                genome = data[[genome]][n],
                                horm = data[[horm]][n])
  }
  cbind.data.frame("METABRIC.ID" = outcomes.t$METABRIC.ID, results)
}

        outcomes.t <- outcomes.t %>%

  # Call modified PREDICT function for all continuous scores
  full_join(., as_tibble(mod_predict_fn(genome = ODx)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = EP)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = MP)),
            by = "METABRIC.ID") %>%
  full_join(., as_tibble(mod_predict_fn(genome = ROR)),
            by = "METABRIC.ID")

erplus <- filter(outcomes.t, ER.Expr == 1, !is.na(PREDICT))
survplus <- with(erplus, Surv(`T`, DeathBreast))

km_erplus <- survfit(survplus ~ 1, data = erplus)
obs_surv_erplus <- summary(km_erplus, times=3652)$surv * 100

# Observed survival at 10y in ER-
erminus <- filter(outcomes.t, ER.Expr == 0, !is.na(PREDICT))
survminus <-  with(erminus, Surv(`T`, DeathBreast))

km_erminus <- survfit(survminus ~ 1, data = erminus)
obs_surv_erminus <- summary(km_erminus, times=3652)$surv * 100

        pred_erplus <- outcomes.t %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

pred_erminus <- outcomes.t %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# Build table of absolute difference in calibration by ER status
calibration_er_t <- bind_rows(pred_erplus, pred_erminus) %>%
  add_column(METABRIC = c(obs_surv_erplus, obs_surv_erminus)) %>%
  t() %>%
  as_tibble() %>%
  rename(ERplus = V1, ERminus = V2) %>%
  mutate(ERplus_delta = ERplus - obs_surv_erplus,
         ERminus_delta = ERminus - obs_surv_erminus) %>%
  mutate(across(everything(), ~ round(.x, digits = 1))) %>%
  dplyr::select(order(colnames(.))) %>%
  add_column(Model = c("PREDICT", models[c(-6,-7)], "METABRIC"),
             .before = TRUE)

newnames <- colnames(calibration_er_t) %>%
    as_tibble() %>%
  mutate(value = case_when(value == "ERminus" ~ paste(value, " (n = ", nrow(erminus), ")", sep = ""),
                           value == "ERplus" ~ paste(value, " (n = ", nrow(erplus), ")", sep = ""),
                           TRUE ~ value))

colnames(calibration_er_t) <- as.vector(newnames$value)

        opt.pos[,i] <- unlist(calibration_er[,5] - calibration_er_t[,5])
        opt.neg[,i] <- unlist(calibration_er[,3] - calibration_er_t[,3])
cat(i, "\n")
        }
rownames(opt.pos) <- calibration_er$Model
rownames(opt.neg) <- calibration_er$Model
apply(opt.pos, 1, mean)
apply(opt.neg, 1, mean)


rm(list = ls()[! (ls() %in% c("outcomes", "models"))])


##### By node status
nodeneg <- outcomes %>%
  filter(Nodes == 0)

nodepos <- outcomes %>%
  filter(Nodes > 0)

# Observed survival at 10y in ER+/LN-
erp_lnm <- filter(nodeneg, ER.Expr == 1, !is.na(PREDICT))
surv_erp_lnm <- with(erp_lnm, Surv(`T`, DeathBreast))

km_erp_lnm <- survfit(surv_erp_lnm ~ 1, data = erp_lnm)
obs_surv_erp_lnm <- km_erp_lnm$surv[which(km_erp_lnm$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/LN+
erp_lnp <- filter(nodepos, ER.Expr == 1, !is.na(PREDICT))
surv_erp_lnp <- with(erp_lnp, Surv(`T`, DeathBreast))

km_erp_lnp <- survfit(surv_erp_lnp ~ 1, data = erp_lnp)
obs_surv_erp_lnp <- km_erp_lnp$surv[which(km_erp_lnp$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/LN-
erm_lnm <- filter(nodeneg, ER.Expr == 0, !is.na(PREDICT))
surv_erm_lnm <- with(erm_lnm, Surv(`T`, DeathBreast))

km_erm_lnm <- survfit(surv_erm_lnm ~ 1, data = erm_lnm)
obs_surv_erm_lnm <- km_erm_lnm$surv[which(km_erm_lnm$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/LN+
erm_lnp <- filter(nodepos, ER.Expr == 0, !is.na(PREDICT))
surv_erm_lnp <- with(erm_lnp, Surv(`T`, DeathBreast))

km_erm_lnp <- survfit(surv_erm_lnp ~ 1, data = erm_lnp)
obs_surv_erm_lnp <- km_erm_lnp$surv[which(km_erm_lnp$time >= 3652)][1] * 100

rm(km_erm_lnm, km_erm_lnp, km_erp_lnm, km_erp_lnp,
   surv_erm_lnm, surv_erm_lnp, surv_erp_lnm, surv_erp_lnp)

# Predicted survival based on modified PREDICT models

# ER+/LN-
pred_erp_lnm <- nodeneg %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/LN+
pred_erp_lnp <- nodepos %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/LN-
pred_erm_lnm <- nodeneg %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/LN+
pred_erm_lnp <- nodepos %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# Build table of absolute difference in calibration by node status
calibration_ln <- bind_rows(pred_erp_lnm, pred_erp_lnp, pred_erm_lnm, pred_erm_lnp) %>%
  add_column(METABRIC = c(obs_surv_erp_lnm, obs_surv_erp_lnp, obs_surv_erm_lnm, obs_surv_erm_lnp)) %>%
  t() %>%
  as_tibble() %>%
  rename(ERplus_LNminus = V1, ERplus_LNplus = V2, ERminus_LNminus = V3, ERminus_LNplus = V4) %>%
  mutate(ERplus_LNminus_delta = ERplus_LNminus - obs_surv_erp_lnm,
         ERplus_LNplus_delta = ERplus_LNplus - obs_surv_erp_lnp,
         ERminus_LNminus_delta = ERminus_LNminus - obs_surv_erm_lnm,
         ERminus_LNplus_delta = ERminus_LNplus - obs_surv_erm_lnp) %>%
  mutate(across(everything(), ~ round(.x, digits = 1))) %>%
  dplyr::select(order(colnames(.))) %>%
  add_column(Model = c("PREDICT", models[c(-6,-7)], "METABRIC"),
             .before = TRUE)

newnames <- colnames(calibration_ln) %>%
  as_tibble() %>%
  mutate(value = case_when(value == "ERplus_LNminus" ~ paste(value, " (n = ", nrow(erp_lnm), ")", sep = ""),
                           value == "ERplus_LNplus" ~ paste(value, " (n = ", nrow(erp_lnp), ")", sep = ""),
                           value == "ERminus_LNminus" ~ paste(value, " (n = ", nrow(erm_lnm), ")", sep = ""),
                           value == "ERminus_LNplus" ~ paste(value, " (n = ", nrow(erm_lnp), ")", sep = ""),
                           TRUE ~ value))

colnames(calibration_ln) <- as.vector(newnames$value)

write_csv(calibration_ln, "calibration_lnstatus.csv")

rm(list = ls()[! (ls() %in% c("outcomes", "models"))])

##### By grade
g1 <- outcomes %>%
  filter(Grade == 1)

g2 <- outcomes %>%
  filter(Grade == 2)

g3 <- outcomes %>%
  filter(Grade == 3)

# Observed survival at 10y in ER+/G1
erp_g1 <- filter(g1, ER.Expr == 1, !is.na(PREDICT))
surv_erp_g1 <- with(erp_g1, Surv(`T`, DeathBreast))

km_erp_g1 <- survfit(surv_erp_g1 ~ 1, data = erp_g1)
obs_surv_erp_g1 <- km_erp_g1$surv[which(km_erp_g1$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/G2
erp_g2 <- filter(g2, ER.Expr == 1, !is.na(PREDICT))
surv_erp_g2 <- with(erp_g2, Surv(`T`, DeathBreast))

km_erp_g2 <- survfit(surv_erp_g2 ~ 1, data = erp_g2)
obs_surv_erp_g2 <- km_erp_g2$surv[which(km_erp_g2$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/G3
erp_g3 <- filter(g3, ER.Expr == 1, !is.na(PREDICT))
surv_erp_g3 <- with(erp_g3, Surv(`T`, DeathBreast))

km_erp_g3 <- survfit(surv_erp_g3 ~ 1, data = erp_g3)
obs_surv_erp_g3 <- km_erp_g3$surv[which(km_erp_g3$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/G1
erm_g1 <- filter(g1, ER.Expr == 0, !is.na(PREDICT))
surv_erm_g1 <- with(erm_g1, Surv(`T`, DeathBreast))

km_erm_g1 <- survfit(surv_erm_g1 ~ 1, data = erm_g1)
obs_surv_erm_g1 <- km_erm_g1$surv[which(km_erm_g1$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/G2
erm_g2 <- filter(g2, ER.Expr == 0, !is.na(PREDICT))
surv_erm_g2 <- with(erm_g2, Surv(`T`, DeathBreast))

km_erm_g2 <- survfit(surv_erm_g2 ~ 1, data = erm_g2)
obs_surv_erm_g2 <- km_erm_g2$surv[which(km_erm_g2$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/G3
erm_g3 <- filter(g3, ER.Expr == 0, !is.na(PREDICT))
surv_erm_g3 <- with(erm_g3, Surv(`T`, DeathBreast))

km_erm_g3 <- survfit(surv_erm_g3 ~ 1, data = erm_g3)
obs_surv_erm_g3 <- km_erm_g3$surv[which(km_erm_g3$time >= 3652)][1] * 100

# Predicted survival based on modified PREDICT models

# ER+/G1
pred_erp_g1 <- g1 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/G2
pred_erp_g2 <- g2 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/G3
pred_erp_g3 <- g3 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/G1
pred_erm_g1 <- g1 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/G2
pred_erm_g2 <- g2 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/G3
pred_erm_g3 <- g3 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# Build table of absolute difference in calibration by grade
calibration_gr <- bind_rows(pred_erp_g1,  pred_erp_g2, pred_erp_g3, pred_erm_g1, pred_erm_g2, pred_erm_g3) %>%
  add_column(METABRIC = c(obs_surv_erp_g1, obs_surv_erp_g2, obs_surv_erp_g3, obs_surv_erm_g1, obs_surv_erm_g2, obs_surv_erm_g3)) %>%
  t() %>%
  as_tibble() %>%
  rename(ERplus_grade1 = V1, ERplus_grade2 = V2, ERplus_grade3 = V3, ERminus_grade1 = V4, ERminus_grade2 = V5, ERminus_grade3 = V6) %>%
  mutate(ERplus_grade1_delta = ERplus_grade1 - obs_surv_erp_g1,
         ERplus_grade2_delta = ERplus_grade2 - obs_surv_erp_g2,
         ERplus_grade3_delta = ERplus_grade3 - obs_surv_erp_g3,
         ERminus_grade1_delta = ERminus_grade1 - obs_surv_erm_g1,
         ERminus_grade2_delta = ERminus_grade2 - obs_surv_erm_g2,
         ERminus_grade3_delta = ERminus_grade3 - obs_surv_erm_g3) %>%
  mutate(across(everything(), ~ round(.x, digits = 1))) %>%
  dplyr::select(order(colnames(.))) %>%
  add_column(Model = c("PREDICT", models[c(-6,-7)], "METABRIC"),
             .before = TRUE)

newnames <- colnames(calibration_gr) %>%
  as_tibble() %>%
  mutate(value = case_when(value == "ERplus_grade1" ~ paste(value, " (n = ", nrow(erp_g1), ")", sep = ""),
                           value == "ERplus_grade2" ~ paste(value, " (n = ", nrow(erp_g2), ")", sep = ""),
                           value == "ERplus_grade3" ~ paste(value, " (n = ", nrow(erp_g3), ")", sep = ""),
                           value == "ERminus_grade1" ~ paste(value, " (n = ", nrow(erm_g1), ")", sep = ""),
                           value == "ERminus_grade2" ~ paste(value, " (n = ", nrow(erm_g2), ")", sep = ""),
                           value == "ERminus_grade3" ~ paste(value, " (n = ", nrow(erm_g3), ")", sep = ""),
                           TRUE ~ value))

colnames(calibration_gr) <- as.vector(newnames$value)

write_csv(calibration_gr, "calibration_grade.csv")

rm(list = ls()[! (ls() %in% c("outcomes", "models"))])

##### By decade of diagnosis
dx80 <- outcomes %>%
  filter(Dx_Year %in% c(1980:1989))

dx90 <- outcomes %>%
  filter(Dx_Year %in% c(1990:1999))

dx00 <- outcomes %>%
  filter(Dx_Year %in% c(2000:2009))

# Observed survival at 10y in ER+/1980s
erp_80 <- filter(dx80, ER.Expr == 1, !is.na(PREDICT))
surv_erp_80 <- with(erp_80, Surv(`T`, DeathBreast))

km_erp_80 <- survfit(surv_erp_80 ~ 1, data = erp_80)
obs_surv_erp_80 <- km_erp_80$surv[which(km_erp_80$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/1990s
erp_90 <- filter(dx90, ER.Expr == 1, !is.na(PREDICT))
surv_erp_90 <- with(erp_90, Surv(`T`, DeathBreast))

km_erp_90 <- survfit(surv_erp_90 ~ 1, data = erp_90)
obs_surv_erp_90 <- km_erp_90$surv[which(km_erp_90$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/2000s
erp_00 <- filter(dx00, ER.Expr == 1, !is.na(PREDICT))
surv_erp_00 <- with(erp_00, Surv(`T`, DeathBreast))

km_erp_00 <- survfit(surv_erp_00 ~ 1, data = erp_00)
obs_surv_erp_00 <- km_erp_00$surv[which(km_erp_00$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/1980s
erm_80 <- filter(dx80, ER.Expr == 0, !is.na(PREDICT))
surv_erm_80 <- with(erm_80, Surv(`T`, DeathBreast))

km_erm_80 <- survfit(surv_erm_80 ~ 1, data = erm_80)
obs_surv_erm_80 <- km_erm_80$surv[which(km_erm_80$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/1990s
erm_90 <- filter(dx90, ER.Expr == 0, !is.na(PREDICT))
surv_erm_90 <- with(erm_90, Surv(`T`, DeathBreast))

km_erm_90 <- survfit(surv_erm_90 ~ 1, data = erm_90)
obs_surv_erm_90 <- km_erm_90$surv[which(km_erm_90$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/2000s
erm_00 <- filter(dx00, ER.Expr == 0, !is.na(PREDICT))
surv_erm_00 <- with(erm_00, Surv(`T`, DeathBreast))

km_erm_00 <- survfit(surv_erm_00 ~ 1, data = erm_00)
obs_surv_erm_00 <- km_erm_00$surv[which(km_erm_00$time >= 3652)][1] * 100

# Predicted survival based on modified PREDICT models

# ER+/1980s
pred_erp_80 <- dx80 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/1990s
pred_erp_90 <- dx90 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/2000s
pred_erp_00 <- dx00 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/1980s
pred_erm_80 <- dx80 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/1990s
pred_erm_90 <- dx90 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/2000s
pred_erm_00 <- dx00 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# Build table of absolute difference in calibration by decade of diagnosis
calibration_yr <- bind_rows(pred_erp_80,  pred_erp_90, pred_erp_00, pred_erm_80, pred_erm_90, pred_erm_00) %>%
  add_column(METABRIC = c(obs_surv_erp_80, obs_surv_erp_90, obs_surv_erp_00, obs_surv_erm_80, obs_surv_erm_90, obs_surv_erm_00)) %>%
  t() %>%
  as_tibble() %>%
  rename(ERplus_1980s = V1, ERplus_1990s = V2, ERplus_2000s = V3, ERminus_1980s = V4, ERminus_1990s = V5, ERminus_2000s = V6) %>%
  mutate(ERplus_1980s_delta = ERplus_1980s - obs_surv_erp_80,
         ERplus_1990s_delta = ERplus_1990s - obs_surv_erp_90,
         ERplus_2000s_delta = ERplus_2000s - obs_surv_erp_00,
         ERminus_1980s_delta = ERminus_1980s - obs_surv_erm_80,
         ERminus_1990s_delta = ERminus_1990s - obs_surv_erm_90,
         ERminus_2000s_delta = ERminus_2000s - obs_surv_erm_00) %>%
  mutate(across(everything(), ~ round(.x, digits = 1))) %>%
  dplyr::select(order(colnames(.))) %>%
  add_column(Model = c("PREDICT", models[c(-6,-7)], "METABRIC"),
             .before = TRUE)

newnames <- colnames(calibration_yr) %>%
  as_tibble() %>%
  mutate(value = case_when(value == "ERplus_1980s" ~ paste(value, " (n = ", nrow(erp_80), ")", sep = ""),
                           value == "ERplus_1990s" ~ paste(value, " (n = ", nrow(erp_90), ")", sep = ""),
                           value == "ERplus_2000s" ~ paste(value, " (n = ", nrow(erp_00), ")", sep = ""),
                           value == "ERminus_1980s" ~ paste(value, " (n = ", nrow(erm_80), ")", sep = ""),
                           value == "ERminus_1990s" ~ paste(value, " (n = ", nrow(erm_90), ")", sep = ""),
                           value == "ERminus_2000s" ~ paste(value, " (n = ", nrow(erm_00), ")", sep = ""),
                           TRUE ~ value))

colnames(calibration_yr) <- as.vector(newnames$value)

write_csv(calibration_yr, "calibration_dxdecade.csv")

rm(list = ls()[! (ls() %in% c("outcomes", "models"))])

##### By cohort
c1 <- outcomes %>%
  filter(Cohort == 1)

c2 <- outcomes %>%
  filter(Cohort == 2)

c3 <- outcomes %>%
  filter(Cohort == 3)

c4 <- outcomes %>%
  filter(Cohort == 4)

c5 <- outcomes %>%
  filter(Cohort == 5)

# Observed survival at 10y in ER+/G1
erp_c1 <- filter(c1, ER.Expr == 1, !is.na(PREDICT))
surv_erp_c1 <- with(erp_c1, Surv(`T`, DeathBreast))

km_erp_c1 <- survfit(surv_erp_c1 ~ 1, data = erp_c1)
obs_surv_erp_c1 <- km_erp_c1$surv[which(km_erp_c1$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/G2
erp_c2 <- filter(c2, ER.Expr == 1, !is.na(PREDICT))
surv_erp_c2 <- with(erp_c2, Surv(`T`, DeathBreast))

km_erp_c2 <- survfit(surv_erp_c2 ~ 1, data = erp_c2)
obs_surv_erp_c2 <- km_erp_c2$surv[which(km_erp_c2$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/G3
erp_c3 <- filter(c3, ER.Expr == 1, !is.na(PREDICT))
surv_erp_c3 <- with(erp_c3, Surv(`T`, DeathBreast))

km_erp_c3 <- survfit(surv_erp_c3 ~ 1, data = erp_c3)
obs_surv_erp_c3 <- km_erp_c3$surv[which(km_erp_c3$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/G4
erp_c4 <- filter(c4, ER.Expr == 1, !is.na(PREDICT))
surv_erp_c4 <- with(erp_c4, Surv(`T`, DeathBreast))

km_erp_c4 <- survfit(surv_erp_c4 ~ 1, data = erp_c4)
obs_surv_erp_c4 <- km_erp_c4$surv[which(km_erp_c4$time >= 3652)][1] * 100

# Observed survival at 10y in ER+/G5
erp_c5 <- filter(c5, ER.Expr == 1, !is.na(PREDICT))
surv_erp_c5 <- with(erp_c5, Surv(`T`, DeathBreast))

km_erp_c5 <- survfit(surv_erp_c5 ~ 1, data = erp_c5)
obs_surv_erp_c5 <- km_erp_c5$surv[which(km_erp_c5$time >= 3652)][1] * 100


# Observed survival at 10y in ER-/G1
erm_c1 <- filter(c1, ER.Expr == 0, !is.na(PREDICT))
surv_erm_c1 <- with(erm_c1, Surv(`T`, DeathBreast))

km_erm_c1 <- survfit(surv_erm_c1 ~ 1, data = erm_c1)
obs_surv_erm_c1 <- km_erm_c1$surv[which(km_erm_c1$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/G2
erm_c2 <- filter(c2, ER.Expr == 0, !is.na(PREDICT))
surv_erm_c2 <- with(erm_c2, Surv(`T`, DeathBreast))

km_erm_c2 <- survfit(surv_erm_c2 ~ 1, data = erm_c2)
obs_surv_erm_c2 <- km_erm_c2$surv[which(km_erm_c2$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/G3
erm_c3 <- filter(c3, ER.Expr == 0, !is.na(PREDICT))
surv_erm_c3 <- with(erm_c3, Surv(`T`, DeathBreast))

km_erm_c3 <- survfit(surv_erm_c3 ~ 1, data = erm_c3)
obs_surv_erm_c3 <- km_erm_c3$surv[which(km_erm_c3$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/G4
erm_c4 <- filter(c4, ER.Expr == 0, !is.na(PREDICT))
surv_erm_c4 <- with(erm_c4, Surv(`T`, DeathBreast))

km_erm_c4 <- survfit(surv_erm_c4 ~ 1, data = erm_c4)
obs_surv_erm_c4 <- km_erm_c4$surv[which(km_erm_c4$time >= 3652)][1] * 100

# Observed survival at 10y in ER-/G5
erm_c5 <- filter(c5, ER.Expr == 0, !is.na(PREDICT))
surv_erm_c5 <- with(erm_c5, Surv(`T`, DeathBreast))

km_erm_c5 <- survfit(surv_erm_c5 ~ 1, data = erm_c5)
obs_surv_erm_c5 <- km_erm_c5$surv[which(km_erm_c5$time >= 3652)][1] * 100

# Predicted survival based on modified PREDICT models

# ER+/G1
pred_erp_c1 <- c1 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/G2
pred_erp_c2 <- c2 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/G3
pred_erp_c3 <- c3 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/G4
pred_erp_c4 <- c4 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER+/G5
pred_erp_c5 <- c5 %>%
  filter(ER.Expr == 1) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))


# ER-/G1
pred_erm_c1 <- c1 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/G2
pred_erm_c2 <- c2 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/G3
pred_erm_c3 <- c3 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/G4
pred_erm_c4 <- c4 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# ER-/G5
pred_erm_c5 <- c5 %>%
  filter(ER.Expr == 0) %>%
  ungroup() %>%
  dplyr::select(ends_with("OS")) %>%
  summarise(across(everything(), ~ mean(.x, na.rm = TRUE)))

# Build table of absolute difference in calibration by grade
calibration_gr <- bind_rows(pred_erp_c1,  pred_erp_c2, pred_erp_c3, pred_erp_c4, pred_erp_c5, pred_erm_c1, pred_erm_c2, pred_erm_c3,
pred_erm_c4, pred_erm_c5) %>%
  add_column(METABRIC = c(obs_surv_erp_c1, obs_surv_erp_c2, obs_surv_erp_c3, obs_surv_erp_c4, obs_surv_erp_c5, obs_surv_erm_c1, obs_surv_erm_c2, obs_surv_erm_c3, obs_surv_erm_c4, obs_surv_erm_c5)) %>%
  t() %>%
  as_tibble() %>%
  rename(ERplus_cohort1 = V1, ERplus_cohort2 = V2, ERplus_cohort3 = V3, ERplus_cohort4 = V4, ERplus_cohort5 = V5, ERminus_cohort1 = V6, ERminus_cohort2 = V7, ERminus_cohort3 = V8, ERminus_cohort4 = V9, ERminus_cohort5 = V10) %>%
  mutate(ERplus_cohort1_delta = ERplus_cohort1 - obs_surv_erp_c1,
         ERplus_cohort2_delta = ERplus_cohort2 - obs_surv_erp_c2,
         ERplus_cohort3_delta = ERplus_cohort3 - obs_surv_erp_c3,
	 ERplus_cohort4_delta = ERplus_cohort4 - obs_surv_erp_c4,
	 ERplus_cohort5_delta = ERplus_cohort5 - obs_surv_erp_c5,
	 ERminus_cohort1_delta = ERminus_cohort1 - obs_surv_erm_c1,
 	 ERminus_cohort2_delta = ERminus_cohort2 - obs_surv_erm_c2,
	 ERminus_cohort3_delta = ERminus_cohort3 - obs_surv_erm_c3,
	 ERminus_cohort4_delta = ERminus_cohort4 - obs_surv_erm_c4,
         ERminus_cohort5_delta = ERminus_cohort5 - obs_surv_erm_c5) %>%
  mutate(across(everything(), ~ round(.x, digits = 1))) %>%
  dplyr::select(order(colnames(.))) %>%
  add_column(Model = c("PREDICT", models[c(-6,-7)], "METABRIC"),
             .before = TRUE)

newnames <- colnames(calibration_gr) %>%
  as_tibble() %>%
  mutate(value = case_when(value == "ERplus_cohort1" ~ paste(value, " (n = ", nrow(erp_c1), ")", sep = ""),
                           value == "ERplus_cohort2" ~ paste(value, " (n = ", nrow(erp_c2), ")", sep = ""),
                           value == "ERplus_cohort3" ~ paste(value, " (n = ", nrow(erp_c3), ")", sep = ""),
                           value == "ERplus_cohort4" ~ paste(value, " (n = ", nrow(erp_c4), ")", sep = ""),
                           value == "ERplus_cohort5" ~ paste(value, " (n = ", nrow(erp_c5), ")", sep = ""),
                           value == "ERminus_cohort1" ~ paste(value, " (n = ", nrow(erm_c1), ")", sep = ""),
                           value == "ERminus_cohort2" ~ paste(value, " (n = ", nrow(erm_c2), ")", sep = ""),
                           value == "ERminus_cohort3" ~ paste(value, " (n = ", nrow(erm_c3), ")", sep = ""),
                           value == "ERminus_cohort4" ~ paste(value, " (n = ", nrow(erm_c4), ")", sep = ""),
                           value == "ERminus_cohort5" ~ paste(value, " (n = ", nrow(erm_c5), ")", sep = ""),
                           TRUE ~ value))

colnames(calibration_gr) <- as.vector(newnames$value)

write_csv(calibration_gr, "calibration_cohort.csv")

rm(list = ls()[! (ls() %in% c("outcomes"))])


                                        # Compare discrimination----
erplus <- outcomes %>%
  filter(ER.Expr == 1) %>%
  dplyr::select(Time = `T`, DeathBreast, starts_with("PREDICT") &
                  ends_with("pi"))

erminus <- outcomes %>%
  filter(ER.Expr == 0) %>%
  dplyr::select(Time = `T`, DeathBreast, starts_with("PREDICT") &
                  ends_with("pi"))

survplus <- with(erplus, Surv(Time, DeathBreast))
survminus <- with(erminus, Surv(Time, DeathBreast))


# Calculation in ER+ patients
mod_plus <- c("PREDICT_ODx_pi", "PREDICT_EP_pi", "PREDICT_MP_pi",
              "PREDICT_ROR_pi")

multiplus <- lapply(erplus[, mod_plus],
                    function(x) coxph(survplus ~ offset(x),
                                      data = erplus))

# Extract and store model results
summary_plus <- lapply(multiplus, summary)
disc_erplus <- as_tibble(cbind(

  # Populate with info from summaries
  model = paste(mod_plus),
  loglik = sapply(summary_plus, function(x) x$loglik),
  c_index = sapply(summary_plus, function(x) x$concordance[6]),
  c_ind_se = sapply(summary_plus, function(x) x$concordance[7]))) %>%

  # Convert to numeric
  mutate(across(.cols = c(2:4), .fns = ~ as.numeric(.))) %>%

  # Neaten up appearance and calculate CIs
  transmute(model = model,
            log_likelihood = signif(loglik, 5),
            concordance_index = signif(c_index, 3),
            c_index_lci = signif(c_index - qnorm(.975) * c_ind_se, 3),
            c_index_uci = signif(c_index + qnorm(.975) * c_ind_se, 3))

rm(summary_plus, multiplus, survplus, erplus)

# ER+ ANOVAs
erplus <- outcomes %>%
  ungroup() %>%
  filter(ER.Expr == 1) %>%
  dplyr::select(Time = `T`, DeathBreast, "PREDICT", starts_with("PREDICT") &
                  ends_with("pi")) %>%
  drop_na()

survplus <- with(erplus, Surv(Time, DeathBreast))

anovaplus <- lapply(erplus[, mod_plus],
                    function(x) anova(coxph(survplus ~ offset(PREDICT),
                                            data = erplus),
                                      coxph(survplus ~ offset(PREDICT) + x,
                                            data = erplus)))

# c-index significance tests
pi <- concordance.index(erplus$PREDICT, surv.time = erplus$Time,
                        surv.event = erplus$DeathBreast, method = "noether")

cstatplus <- lapply(erplus[, mod_plus], function(x)
  cindex.comp(pi, concordance.index(x, surv.time = erplus$Time,
                                    surv.event = erplus$DeathBreast,
                                    method = "noether")))

rm(anovaplus, cstatplus, pi)

# Calculation in ER- patients
multiminus <- lapply(erminus[, mod_plus],
                     function(x) coxph(survminus ~ offset(x),
                                       data = erminus))

# Extract and store model results
disc_erminus <- as_tibble(cbind(

  # Populate with info from summaries
  model = mod_plus,
  loglik = multiminus$PREDICT_EP_pi$loglik,
  c_index = multiminus$PREDICT_EP_pi$concordance[6],
  c_ind_se = multiminus$PREDICT_EP_pi$concordance[7])) %>%

  # Convert to numeric
  mutate(across(.cols = c(2:4), .fns = ~ as.numeric(.))) %>%

  # Neaten up appearance and calculate CIs
  transmute(model = model,
            log_likelihood = signif(loglik, 5),
            concordance_index = signif(c_index, 3),
            c_index_lci = signif(c_index - qnorm(.975) * c_ind_se, 3),
            c_index_uci = signif(c_index + qnorm(.975) * c_ind_se, 3))

# ER- ANOVAs
erminus <- outcomes %>%
  ungroup() %>%
  filter(ER.Expr == 0) %>%
  dplyr::select(Time = `T`, DeathBreast, "PREDICT", starts_with("PREDICT") &
                  ends_with("pi")) %>%
  drop_na()

survminus <- with(erminus, Surv(Time, DeathBreast))

anovaminus <- lapply(erminus[, mod_plus],
                     function(x) anova(coxph(survminus ~ offset(PREDICT),
                                             data = erminus),
                                       coxph(survminus ~ offset(PREDICT) + x,
                                             data = erminus)))

# c-index significance tests
pi <- concordance.index(erminus$PREDICT, surv.time = erminus$Time,
                        surv.event = erminus$DeathBreast, method = "noether")

cstatminus <- lapply(erminus[, mod_plus], function(x)
  cindex.comp(pi, concordance.index(x, surv.time = erminus$Time,
                                    surv.event = erminus$DeathBreast,
                                    method = "noether")))

# Produce single table of multivariable models and write to .csv
discrimination <- bind_rows(disc_erplus, disc_erminus, .id = "ER") %>%
  mutate(ER = case_when(ER == 1 ~ "Positive", ER == 2 ~ "Negative"))

write_csv(discrimination, "discrimination.csv")

rm(disc_erminus, disc_erplus, multiminus, multivar_erminus, mod_minus,
   survminus, discrimination, erplus, erminus, cstatminus, pi, anovaminus,
   mod_plus, multivar_erplus, survplus)

# Compare reclassification----
classes <- outcomes %>%
  dplyr::select(ends_with("deltaCT")) %>%

  # Create classes for original PREDICT model (as a sanity check)
  mutate(baseline_class = case_when(PREDICT_deltaCT > 5 ~ 3,
                                    (PREDICT_deltaCT <= 5 & PREDICT_deltaCT >= 3) ~ 2,
                                    PREDICT_deltaCT < 3 ~ 1)) %>%

  # Mutate all groups into classes based on clinical decision thresholds
  mutate(across(starts_with("PREDICT"),
                ~ case_when (.x > 5 ~ 3, # High benefit
                             (.x <= 5 & . >= 3) ~ 2, # Intermediate benefit
                             .x < 3 ~ 1))) %>% # Low benefit
  rename_with(~ str_replace(.x, "_deltaCT", "")) %>%
  rename_with(~ str_replace(.x, "PREDICT_", ""))

# Original PREDICT model sanity check
table(classes$baseline_class == classes$PREDICT)

# Calculate reclassifications per signature
ODx_reclass <- classes %>%
  ungroup() %>%
  group_by(PREDICT) %>%
  count(ODx) %>%
  drop_na() %>%
  pivot_wider(names_from = ODx, names_prefix = "ODx", values_from = n)

EP_reclass <- classes %>%
  ungroup() %>%
  group_by(PREDICT) %>%
  count(EP) %>%
  drop_na() %>%
  pivot_wider(names_from = EP, names_prefix = "EP", values_from = n)

MP_reclass <- classes %>%
  ungroup() %>%
  group_by(PREDICT) %>%
  count(MP) %>%
  drop_na() %>%
  pivot_wider(names_from = MP, names_prefix = "MP", values_from = n)

ROR_reclass <- classes %>%
  ungroup() %>%
  group_by(PREDICT) %>%
  count(ROR) %>%
  drop_na() %>%
  pivot_wider(names_from = ROR, names_prefix = "ROR", values_from = n)

reclassifications <- ODx_reclass %>%
  full_join(EP_reclass) %>%
  full_join(MP_reclass) %>%
  full_join(ROR_reclass)

# Example calculations into and out of intermediate
table(classes$baseline_class != classes$ODx)
table(classes$baseline_class == 2 & classes$baseline_class != classes$ROR) # Out of intermediate
table(classes$ROR == 2 & classes$baseline_class != classes$ROR) # Into intermediate

write_csv(reclassifications, "reclassifications.csv")

# Create supplementary figures ----

orange <- "#e8ab5f"
green <- "#a1ad62"

background_diff <- "#eef0e2"

gray <- "#bbbbbb"
light_gray <- "#f4f4f4"

# ER status

er_full <- read_csv("calibration_erstatus.csv") %>%
  rename_with(.cols = contains("ER"), .fn = ~ case_when(str_detect(., "ERplus") & str_detect(., "\\(n") ~ "ERplus_predicted",
                                                str_detect(., "ERminus") & str_detect(., "\\(n") ~ "ERminus_predicted",
                                                TRUE ~ .
                                                )) %>%
  mutate(ERminus_observed = ERminus_predicted[Model == "METABRIC"],
         ERplus_observed = ERplus_predicted[Model == "METABRIC"]) %>%
  pivot_longer(cols = -c(Model, ERplus_delta, ERminus_delta), names_to = c("ERstatus", "type"), names_sep = "_") %>%
  mutate(Model = as.factor(Model),
         value = value/100,
         delta = case_when(ERstatus == "ERplus" ~ ERplus_delta/100,
                      ERstatus == "ERminus" ~ ERminus_delta/100,
                      TRUE ~ value)) %>%

  # Remove METABRIC
  filter(Model != "METABRIC") %>%
  mutate(Model = fct_relevel(Model, "ROR", "MP", "EP", "ODx", "PREDICT")) %>%
  dplyr::select(Model, ERstatus, type, value, delta) %>%
  mutate(Model = fct_recode(Model, "PREDICT\n + Prosigna" = "ROR", "PREDICT\n + MammaPrint" = "MP", "PREDICT\n + EndoPredict" = "EP",
                            "PREDICT\n + Oncotype Dx" = "ODx"))

# Produce dumbbell plots
for (i in 1:2) {
iteration <- c("ERplus", "ERminus")

er <- er_full %>%
  filter(ERstatus == iteration[i])

plot <- ggplot(
  data = er,
  mapping = aes(
    # Y Axis
    y=Model,
    # X Axis
    x=value,
    # If difference is zero, put it into a third group
    color = ifelse(delta == 0, "zero", type))) +
  # Plot lines between points, by model
  geom_line(
    mapping = aes(group = Model),
    color = gray,
    size = 2.5
  ) +
  geom_point(
    # Dot size
    size=4,
    # dot type
    pch = 19
  ) +
  # Add % for each point
  geom_text(
    # Font size
    size = 4,
    # Set text a little below the dots
    nudge_y = -0.35,
    nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
    mapping =
      aes(
        label =
          # If type is PREDICT (the first one), plot numbers with %
          ifelse(Model == "PREDICT",
                 # Value_if_True:
                 paste0(as.character(round(value*100,1)),"%"),
                 # Value_if_False
                 paste0(as.character(round(value*100,1)))
          ),
        color = ifelse(delta == 0, "zero", type)
      )
  ) +
  # Add "Legend" above PREDICT (the first one)
  geom_text(
    # Font size
    size = 4,
    # Bold face
    fontface = "bold",
    # Set text a little above the dots
    nudge_y = 0.6,
    nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
    mapping =
      aes(
        label =
          # If PREDICT, plot legend
          ifelse(Model == "PREDICT",
                 # Value_if_True:
                 ifelse(type == "predicted",
                        # Value_if_True:
                        "Predicted",
                        # Value_if_False:
                        "Observed"
                 ),
                 # Value_if_False
                 ""
          ),
        color = ifelse(delta == 0, "zero", type)
      )
  ) +
  # Change dot colors
  scale_color_manual(
    values = c(orange, green, "gray")
  ) +
  # Change scale x axis
  scale_x_continuous(
    # Set limits to 0 and 1.2
    limits = c(0,1.2),
    # Show tick marks at every 25%
    breaks = seq(0,1,.25),
    # Change scale to percent
    labels = scales::percent
  ) +
  # Expand y axis scale so that the legend can fit
  scale_y_discrete(
    expand = expand_scale(add=c(0.65,1))
  ) +
  # Add white rectangle to set the area where the values of the differences will be
  geom_rect(
    mapping = aes(xmin = 1.01, xmax = Inf , ymin = -Inf, ymax = Inf),
    fill = "white",
    color = "white"
  ) +
  # Add rectangle with correct background color for the differences
  geom_rect(
    mapping = aes(xmin = 1.05, xmax = 1.15 , ymin = -Inf, ymax = Inf),
    fill = background_diff,
    color = background_diff
  ) +
  # Add Differences values
  geom_text(
    # Bold face
    fontface = "bold",
    # Font size
    size = 4,
    # Font Color
    colour = "black",
    # Position
    mapping =
      aes(
        x = 1.1,
        y = Model,
        label =
          # To avoid duplicate values, plot empty text for the first group and
          # plot the difference only for the Ages_18_34 group.
          ifelse(type == "predicted",
                 # Value_if_True
                 "",
                 #Value_if_False
                 # If the difference is equal to zero, do not put any signal.
                 # Otherwise, if Positive, put the + sign on the front.
                 ifelse(delta == 0,
                        # Value_if_True:
                        paste0(as.character(round(delta*100,1))),
                        # Value_if_False
                        ifelse(delta > 0,
                               # Value_if_True
                               paste0("+",as.character(round(delta*100,1))),
                               # Value_if_False
                               paste0(as.character(round(delta*100,1)))
                        )
                 )
          )
      )
  ) +
  # Insert Title of Differences
  geom_text(
    fontface = "bold",
    size = 4,
    colour = "gray",
    # Set text a little above the dots
    nudge_y = 0.6,
    # Position
    mapping =
      aes(
        x = 1.1,
        y = Model,
        label =
          # If PREDICT, plot values
          ifelse(Model == "PREDICT",
                 # Value_if_True
                 "Δ",
                 #Value_if_False
                 ""
          )
      )
  ) +
  # Axis Labels
  labs(
    x = "",
    y = ""
  ) +
  # Change background, General font size, and other things
  theme(
    # Change font color and text for all text outside geom_text
    text = element_text(color = "#4e4d47", size = 14),

    # Country names in bold face
    axis.text.y = element_text(face = "bold"),

    # Add space between x axis text and plot
    axis.text.x = element_text(vjust = -0.75),

    # Do not show tick marks
    axis.ticks = element_blank(),

    # Delete original legend
    legend.position = "none",

    # White background
    panel.background = element_blank(),

    # Country (y Axis) Lines
    panel.grid.major.y = element_line(colour = light_gray, size = 1),

    # Change Title Font
    plot.title = element_text(face = "bold", size = 16))

ggsave(plot = plot, filename = paste("ERstatus_", iteration[i], "dumbbellplot.png", sep = ""), width = 3043, height = 1165, units = "px")
}

# Lymph nodes
nodes <- read_csv("calibration_lnstatus.csv") %>%

  rename_with(.cols = contains("ER"), .fn = ~ case_when(str_detect(., "ERplus") & str_detect(., "LNminus")& str_detect(., "\\(n") ~ "ERplus_LNminus_predicted",
                                                        str_detect(., "ERplus") & str_detect(., "LNplus")& str_detect(., "\\(n") ~ "ERplus_LNplus_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "LNminus")& str_detect(., "\\(n") ~ "ERminus_LNminus_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "LNplus")& str_detect(., "\\(n") ~ "ERminus_LNplus_predicted",
                                                        TRUE ~ .
  )) %>%
  mutate(ERplus_LNminus_observed = ERplus_LNminus_predicted[Model == "METABRIC"],
         ERplus_LNplus_observed = ERplus_LNplus_predicted[Model == "METABRIC"],
         ERminus_LNminus_observed = ERminus_LNminus_predicted[Model == "METABRIC"],
         ERminus_LNplus_observed = ERminus_LNplus_predicted[Model == "METABRIC"]) %>%
  pivot_longer(cols = -c(Model, contains("delta")), names_to = c("ERstatus", "nodes", "type"), names_sep = "_") %>%
  mutate(Model = as.factor(Model),
         delta = case_when(ERstatus == "ERplus" & nodes == "LNminus" ~ ERplus_LNminus_delta/100,
                           ERstatus == "ERplus" & nodes == "LNplus" ~ ERplus_LNplus_delta/100,
                           ERstatus == "ERminus" & nodes == "LNminus" ~ ERminus_LNminus_delta/100,
                           ERstatus == "ERminus" & nodes == "LNplus" ~ ERminus_LNplus_delta/100),
         value = value/100) %>%

  # Remove METABRIC
  filter(Model != "METABRIC") %>%
  mutate(Model = fct_relevel(Model, "ROR", "MP", "EP", "ODx", "PREDICT")) %>%
  dplyr::select(Model, ERstatus, nodes, type, value, delta) %>%
  mutate(Model = fct_recode(Model, "PREDICT\n + Prosigna" = "ROR", "PREDICT\n + MammaPrint" = "MP", "PREDICT\n + EndoPredict" = "EP",
                            "PREDICT\n + Oncotype Dx" = "ODx"))

# Produce dumbbell plots
for (i in 1:2) {
  iteration <- c("ERplus", "ERminus")

  for (j in 1:2) {

    nodestat <- c("LNminus", "LNplus")

  er <- nodes %>%
    filter(ERstatus == iteration[i] & nodes == nodestat[j])

  plot <- ggplot(
    data = er,
    mapping = aes(
      # Y Axis
      y=Model,
      # X Axis
      x=value,
      # If difference is zero, put it into a third group
      color = ifelse(delta == 0, "zero", type))) +
    # Plot lines between points, by model
    geom_line(
      mapping = aes(group = Model),
      color = gray,
      size = 2.5
    ) +
    geom_point(
      # Dot size
      size=4,
      # dot type
      pch = 19
    ) +
    # Add % for each point
    geom_text(
      # Font size
      size = 4,
      # Set text a little below the dots
      nudge_y = -0.35,
      nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
      mapping =
        aes(
          label =
            # If type is PREDICT (the first one), plot numbers with %
            ifelse(Model == "PREDICT",
                   # Value_if_True:
                   paste0(as.character(round(value*100,1)),"%"),
                   # Value_if_False
                   paste0(as.character(round(value*100,1)))
            ),
          color = ifelse(delta == 0, "zero", type)
        )
    ) +
    # Add "Legend" above PREDICT (the first one)
    geom_text(
      # Font size
      size = 4,
      # Bold face
      fontface = "bold",
      # Set text a little above the dots
      nudge_y = 0.6,
      nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
      mapping =
        aes(
          label =
            # If PREDICT, plot legend
            ifelse(Model == "PREDICT",
                   # Value_if_True:
                   ifelse(type == "predicted",
                          # Value_if_True:
                          "Predicted",
                          # Value_if_False:
                          "Observed"
                   ),
                   # Value_if_False
                   ""
            ),
          color = ifelse(delta == 0, "zero", type)
        )
    ) +
    # Change dot colors
    scale_color_manual(
      values = c(orange, green, "gray")
    ) +
    # Change scale x axis
    scale_x_continuous(
      # Set limits to 0 and 1.2
      limits = c(0,1.2),
      # Show tick marks at every 25%
      breaks = seq(0,1,.25),
      # Change scale to percent
      labels = scales::percent
    ) +
    # Expand y axis scale so that the legend can fit
    scale_y_discrete(
      expand = expand_scale(add=c(0.65,1))
    ) +
    # Add white rectangle to set the area where the values of the differences will be
    geom_rect(
      mapping = aes(xmin = 1.01, xmax = Inf , ymin = -Inf, ymax = Inf),
      fill = "white",
      color = "white"
    ) +
    # Add rectangle with correct background color for the differences
    geom_rect(
      mapping = aes(xmin = 1.05, xmax = 1.15 , ymin = -Inf, ymax = Inf),
      fill = background_diff,
      color = background_diff
    ) +
    # Add Differences values
    geom_text(
      # Bold face
      fontface = "bold",
      # Font size
      size = 4,
      # Font Color
      colour = "black",
      # Position
      mapping =
        aes(
          x = 1.1,
          y = Model,
          label =
            # To avoid duplicate values, plot empty text for the first group and
            # plot the difference only for the Ages_18_34 group.
            ifelse(type == "predicted",
                   # Value_if_True
                   "",
                   #Value_if_False
                   # If the difference is equal to zero, do not put any signal.
                   # Otherwise, if Positive, put the + sign on the front.
                   ifelse(delta == 0,
                          # Value_if_True:
                          paste0(as.character(round(delta*100,1))),
                          # Value_if_False
                          ifelse(delta > 0,
                                 # Value_if_True
                                 paste0("+",as.character(round(delta*100,1))),
                                 # Value_if_False
                                 paste0(as.character(round(delta*100,1)))
                          )
                   )
            )
        )
    ) +
    # Insert Title of Differences
    geom_text(
      fontface = "bold",
      size = 4,
      colour = "gray",
      # Set text a little above the dots
      nudge_y = 0.6,
      # Position
      mapping =
        aes(
          x = 1.1,
          y = Model,
          label =
            # If PREDICT, plot values
            ifelse(Model == "PREDICT",
                   # Value_if_True
                   "Δ",
                   #Value_if_False
                   ""
            )
        )
    ) +
    # Axis Labels
    labs(
      x = "",
      y = ""
    ) +
    # Change background, General font size, and other things
    theme(
      # Change font color and text for all text outside geom_text
      text = element_text(color = "#4e4d47", size = 14),

      # Country names in bold face
      axis.text.y = element_text(face = "bold"),

      # Add space between x axis text and plot
      axis.text.x = element_text(vjust = -0.75),

      # Do not show tick marks
      axis.ticks = element_blank(),

      # Delete original legend
      legend.position = "none",

      # White background
      panel.background = element_blank(),

      # Country (y Axis) Lines
      panel.grid.major.y = element_line(colour = light_gray, size = 1),

      # Change Title Font
      plot.title = element_text(face = "bold", size = 16))

  ggsave(plot = plot, filename = paste(iteration[i], nodestat[j], "dumbbellplot.png", sep = "_"), width = 3043, height = 1165, units = "px")
  }
}

# Grade

grade <- read_csv("calibration_grade.csv") %>%

  rename_with(.cols = contains("ER"), .fn = ~ case_when(str_detect(., "ERplus") & str_detect(., "grade1")& str_detect(., "\\(n") ~ "ERplus_grade1_predicted",
                                                        str_detect(., "ERplus") & str_detect(., "grade2")& str_detect(., "\\(n") ~ "ERplus_grade2_predicted",
                                                        str_detect(., "ERplus") & str_detect(., "grade3")& str_detect(., "\\(n") ~ "ERplus_grade3_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "grade1")& str_detect(., "\\(n") ~ "ERminus_grade1_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "grade2")& str_detect(., "\\(n") ~ "ERminus_grade2_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "grade3")& str_detect(., "\\(n") ~ "ERminus_grade3_predicted",
                                                        TRUE ~ .)) %>%
  mutate(ERplus_grade1_observed = ERplus_grade1_predicted[Model == "METABRIC"],
         ERplus_grade2_observed = ERplus_grade2_predicted[Model == "METABRIC"],
         ERplus_grade3_observed = ERplus_grade3_predicted[Model == "METABRIC"],
         ERminus_grade1_observed = ERminus_grade1_predicted[Model == "METABRIC"],
         ERminus_grade2_observed = ERminus_grade2_predicted[Model == "METABRIC"],
         ERminus_grade3_observed = ERminus_grade3_predicted[Model == "METABRIC"]
         ) %>%
  pivot_longer(cols = -c(Model, contains("delta")), names_to = c("ERstatus", "grade", "type"), names_sep = "_") %>%
  mutate(Model = as.factor(Model),
         delta = case_when(ERstatus == "ERplus" & grade == "grade1" ~ ERplus_grade1_delta/100,
                           ERstatus == "ERplus" & grade == "grade2" ~ ERplus_grade2_delta/100,
                           ERstatus == "ERplus" & grade == "grade3" ~ ERplus_grade3_delta/100,
                           ERstatus == "ERminus" & grade == "grade1" ~ ERminus_grade1_delta/100,
                           ERstatus == "ERminus" & grade == "grade2" ~ ERminus_grade2_delta/100,
                           ERstatus == "ERminus" & grade == "grade3" ~ ERminus_grade3_delta/100),
         value = value/100) %>%

  # Remove METABRIC
  filter(Model != "METABRIC") %>%
  mutate(Model = fct_relevel(Model, "ROR", "MP", "EP", "ODx", "PREDICT")) %>%
  dplyr::select(Model, ERstatus, grade, type, value, delta) %>%
  mutate(Model = fct_recode(Model, "PREDICT\n + Prosigna" = "ROR", "PREDICT\n + MammaPrint" = "MP", "PREDICT\n + EndoPredict" = "EP",
                            "PREDICT\n + Oncotype Dx" = "ODx"))

# Produce dumbbell plots
for (i in 1:2) {
  iteration <- c("ERplus", "ERminus")

  for (j in 1:3) {

    gradestat <- c("grade1", "grade2", "grade3")

    er <- grade %>%
      filter(ERstatus == iteration[i] & grade == gradestat[j])

    plot <- ggplot(
      data = er,
      mapping = aes(
        # Y Axis
        y=Model,
        # X Axis
        x=value,
        # If difference is zero, put it into a third group
        color = ifelse(delta == 0, "zero", type))) +
      # Plot lines between points, by model
      geom_line(
        mapping = aes(group = Model),
        color = gray,
        size = 2.5
      ) +
      geom_point(
        # Dot size
        size=4,
        # dot type
        pch = 19
      ) +
      # Add % for each point
      geom_text(
        # Font size
        size = 4,
        # Set text a little below the dots
        nudge_y = -0.35,
        nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
        mapping =
          aes(
            label =
              # If type is PREDICT (the first one), plot numbers with %
              ifelse(Model == "PREDICT",
                     # Value_if_True:
                     paste0(as.character(round(value*100,1)),"%"),
                     # Value_if_False
                     paste0(as.character(round(value*100,1)))
              ),
            color = ifelse(delta == 0, "zero", type)
          )
      ) +
      # Add "Legend" above PREDICT (the first one)
      geom_text(
        # Font size
        size = 4,
        # Bold face
        fontface = "bold",
        # Set text a little above the dots
        nudge_y = 0.6,
        nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
        mapping =
          aes(
            label =
              # If PREDICT, plot legend
              ifelse(Model == "PREDICT",
                     # Value_if_True:
                     ifelse(type == "predicted",
                            # Value_if_True:
                            "Predicted",
                            # Value_if_False:
                            "Observed"
                     ),
                     # Value_if_False
                     ""
              ),
            color = ifelse(delta == 0, "zero", type)
          )
      ) +
      # Change dot colors
      scale_color_manual(
        values = c(orange, green, "gray")
      ) +
      # Change scale x axis
      scale_x_continuous(
        # Set limits to 0 and 1.2
        limits = c(0,1.2),
        # Show tick marks at every 25%
        breaks = seq(0,1,.25),
        # Change scale to percent
        labels = scales::percent
      ) +
      # Expand y axis scale so that the legend can fit
      scale_y_discrete(
        expand = expand_scale(add=c(0.65,1))
      ) +
      # Add white rectangle to set the area where the values of the differences will be
      geom_rect(
        mapping = aes(xmin = 1.01, xmax = Inf , ymin = -Inf, ymax = Inf),
        fill = "white",
        color = "white"
      ) +
      # Add rectangle with correct background color for the differences
      geom_rect(
        mapping = aes(xmin = 1.05, xmax = 1.15 , ymin = -Inf, ymax = Inf),
        fill = background_diff,
        color = background_diff
      ) +
      # Add Differences values
      geom_text(
        # Bold face
        fontface = "bold",
        # Font size
        size = 4,
        # Font Color
        colour = "black",
        # Position
        mapping =
          aes(
            x = 1.1,
            y = Model,
            label =
              # To avoid duplicate values, plot empty text for the first group and
              # plot the difference only for the Ages_18_34 group.
              ifelse(type == "predicted",
                     # Value_if_True
                     "",
                     #Value_if_False
                     # If the difference is equal to zero, do not put any signal.
                     # Otherwise, if Positive, put the + sign on the front.
                     ifelse(delta == 0,
                            # Value_if_True:
                            paste0(as.character(round(delta*100,1))),
                            # Value_if_False
                            ifelse(delta > 0,
                                   # Value_if_True
                                   paste0("+",as.character(round(delta*100,1))),
                                   # Value_if_False
                                   paste0(as.character(round(delta*100,1)))
                            )
                     )
              )
          )
      ) +
      # Insert Title of Differences
      geom_text(
        fontface = "bold",
        size = 4,
        colour = "gray",
        # Set text a little above the dots
        nudge_y = 0.6,
        # Position
        mapping =
          aes(
            x = 1.1,
            y = Model,
            label =
              # If PREDICT, plot values
              ifelse(Model == "PREDICT",
                     # Value_if_True
                     "Δ",
                     #Value_if_False
                     ""
              )
          )
      ) +
      # Axis Labels
      labs(
        x = "",
        y = ""
      ) +
      # Change background, General font size, and other things
      theme(
        # Change font color and text for all text outside geom_text
        text = element_text(color = "#4e4d47", size = 14),

        # Country names in bold face
        axis.text.y = element_text(face = "bold"),

        # Add space between x axis text and plot
        axis.text.x = element_text(vjust = -0.75),

        # Do not show tick marks
        axis.ticks = element_blank(),

        # Delete original legend
        legend.position = "none",

        # White background
        panel.background = element_blank(),

        # Country (y Axis) Lines
        panel.grid.major.y = element_line(colour = light_gray, size = 1),

        # Change Title Font
        plot.title = element_text(face = "bold", size = 16))

    ggsave(plot = plot, filename = paste(iteration[i], gradestat[j], "dumbbellplot.png", sep = "_"), width = 3043, height = 1165, units = "px")
  }
}

# Decade
decade <- read_csv("calibration_dxdecade.csv") %>%

rename_with(.cols = contains("ER"), .fn = ~ case_when(str_detect(., "ERplus") & str_detect(., "1980s")& str_detect(., "\\(n") ~ "ERplus_1980s_predicted",
                                                      str_detect(., "ERplus") & str_detect(., "1990s")& str_detect(., "\\(n") ~ "ERplus_1990s_predicted",
                                                      str_detect(., "ERplus") & str_detect(., "2000s")& str_detect(., "\\(n") ~ "ERplus_2000s_predicted",
                                                      str_detect(., "ERminus") & str_detect(., "1980s")& str_detect(., "\\(n") ~ "ERminus_1980s_predicted",
                                                      str_detect(., "ERminus") & str_detect(., "1990s")& str_detect(., "\\(n") ~ "ERminus_1990s_predicted",
                                                      str_detect(., "ERminus") & str_detect(., "2000s")& str_detect(., "\\(n") ~ "ERminus_2000s_predicted",
                                                      TRUE ~ .)) %>%
  mutate(ERplus_1980s_observed = ERplus_1980s_predicted[Model == "METABRIC"],
         ERplus_1990s_observed = ERplus_1990s_predicted[Model == "METABRIC"],
         ERplus_2000s_observed = ERplus_2000s_predicted[Model == "METABRIC"],
         ERminus_1980s_observed = ERminus_1980s_predicted[Model == "METABRIC"],
         ERminus_1990s_observed = ERminus_1990s_predicted[Model == "METABRIC"],
         ERminus_2000s_observed = ERminus_2000s_predicted[Model == "METABRIC"]
  ) %>%
  pivot_longer(cols = -c(Model, contains("delta")), names_to = c("ERstatus", "yr", "type"), names_sep = "_") %>%
  mutate(Model = as.factor(Model),
         delta = case_when(ERstatus == "ERplus" & yr == "1980s" ~ ERplus_1980s_delta/100,
                           ERstatus == "ERplus" & yr == "1990s" ~ ERplus_1990s_delta/100,
                           ERstatus == "ERplus" & yr == "2000s" ~ ERplus_2000s_delta/100,
                           ERstatus == "ERminus" & yr == "1980s" ~ ERminus_1980s_delta/100,
                           ERstatus == "ERminus" & yr == "1990s" ~ ERminus_1990s_delta/100,
                           ERstatus == "ERminus" & yr == "2000s" ~ ERminus_2000s_delta/100),
         value = value/100) %>%

  # Remove METABRIC
  filter(Model != "METABRIC") %>%
  mutate(Model = fct_relevel(Model, "ROR", "MP", "EP", "ODx", "PREDICT")) %>%
  dplyr::select(Model, ERstatus, yr, type, value, delta) %>%
  mutate(Model = fct_recode(Model, "PREDICT\n + Prosigna" = "ROR", "PREDICT\n + MammaPrint" = "MP", "PREDICT\n + EndoPredict" = "EP",
                            "PREDICT\n + Oncotype Dx" = "ODx"))

# Produce dumbbell plots
for (i in 1:2) {
  iteration <- c("ERplus", "ERminus")

  for (j in 1:3) {

    decadestat <- c("1980s", "1990s", "2000s")

    er <- decade %>%
      filter(ERstatus == iteration[i] & yr == decadestat[j])

    plot <- ggplot(
      data = er,
      mapping = aes(
        # Y Axis
        y=Model,
        # X Axis
        x=value,
        # If difference is zero, put it into a third group
        color = ifelse(delta == 0, "zero", type))) +
      # Plot lines between points, by model
      geom_line(
        mapping = aes(group = Model),
        color = gray,
        size = 2.5
      ) +
      geom_point(
        # Dot size
        size=4,
        # dot type
        pch = 19
      ) +
      # Add % for each point
      geom_text(
        # Font size
        size = 4,
        # Set text a little below the dots
        nudge_y = -0.35,
        nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
        mapping =
          aes(
            label =
              # If type is PREDICT (the first one), plot numbers with %
              ifelse(Model == "PREDICT",
                     # Value_if_True:
                     paste0(as.character(round(value*100,1)),"%"),
                     # Value_if_False
                     paste0(as.character(round(value*100,1)))
              ),
            color = ifelse(delta == 0, "zero", type)
          )
      ) +
      # Add "Legend" above PREDICT (the first one)
      geom_text(
        # Font size
        size = 4,
        # Bold face
        fontface = "bold",
        # Set text a little above the dots
        nudge_y = 0.6,
        nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
        mapping =
          aes(
            label =
              # If PREDICT, plot legend
              ifelse(Model == "PREDICT",
                     # Value_if_True:
                     ifelse(type == "predicted",
                            # Value_if_True:
                            "Predicted",
                            # Value_if_False:
                            "Observed"
                     ),
                     # Value_if_False
                     ""
              ),
            color = ifelse(delta == 0, "zero", type)
          )
      ) +
      # Change dot colors
      scale_color_manual(
        values = c(orange, green, "gray")
      ) +
      # Change scale x axis
      scale_x_continuous(
        # Set limits to 0 and 1.2
        limits = c(0,1.2),
        # Show tick marks at every 25%
        breaks = seq(0,1,.25),
        # Change scale to percent
        labels = scales::percent
      ) +
      # Expand y axis scale so that the legend can fit
      scale_y_discrete(
        expand = expand_scale(add=c(0.65,1))
      ) +
      # Add white rectangle to set the area where the values of the differences will be
      geom_rect(
        mapping = aes(xmin = 1.01, xmax = Inf , ymin = -Inf, ymax = Inf),
        fill = "white",
        color = "white"
      ) +
      # Add rectangle with correct background color for the differences
      geom_rect(
        mapping = aes(xmin = 1.05, xmax = 1.15 , ymin = -Inf, ymax = Inf),
        fill = background_diff,
        color = background_diff
      ) +
      # Add Differences values
      geom_text(
        # Bold face
        fontface = "bold",
        # Font size
        size = 4,
        # Font Color
        colour = "black",
        # Position
        mapping =
          aes(
            x = 1.1,
            y = Model,
            label =
              # To avoid duplicate values, plot empty text for the first group and
              # plot the difference only for the Ages_18_34 group.
              ifelse(type == "predicted",
                     # Value_if_True
                     "",
                     #Value_if_False
                     # If the difference is equal to zero, do not put any signal.
                     # Otherwise, if Positive, put the + sign on the front.
                     ifelse(delta == 0,
                            # Value_if_True:
                            paste0(as.character(round(delta*100,1))),
                            # Value_if_False
                            ifelse(delta > 0,
                                   # Value_if_True
                                   paste0("+",as.character(round(delta*100,1))),
                                   # Value_if_False
                                   paste0(as.character(round(delta*100,1)))
                            )
                     )
              )
          )
      ) +
      # Insert Title of Differences
      geom_text(
        fontface = "bold",
        size = 4,
        colour = "gray",
        # Set text a little above the dots
        nudge_y = 0.6,
        # Position
        mapping =
          aes(
            x = 1.1,
            y = Model,
            label =
              # If PREDICT, plot values
              ifelse(Model == "PREDICT",
                     # Value_if_True
                     "Δ",
                     #Value_if_False
                     ""
              )
          )
      ) +
      # Axis Labels
      labs(
        x = "",
        y = ""
      ) +
      # Change background, General font size, and other things
      theme(
        # Change font color and text for all text outside geom_text
        text = element_text(color = "#4e4d47", size = 14),

        # Country names in bold face
        axis.text.y = element_text(face = "bold"),

        # Add space between x axis text and plot
        axis.text.x = element_text(vjust = -0.75),

        # Do not show tick marks
        axis.ticks = element_blank(),

        # Delete original legend
        legend.position = "none",

        # White background
        panel.background = element_blank(),

        # Country (y Axis) Lines
        panel.grid.major.y = element_line(colour = light_gray, size = 1),

        # Change Title Font
        plot.title = element_text(face = "bold", size = 16))

    ggsave(plot = plot, filename = paste(iteration[i], decadestat[j], "dumbbellplot.png", sep = "_"), width = 3043, height = 1165, units = "px")
  }
}

# Cohort
cohort <- read_csv("calibration_cohort.csv") %>%

  rename_with(.cols = contains("ER"), .fn = ~ case_when(str_detect(., "ERplus") & str_detect(., "cohort1")& str_detect(., "\\(n") ~ "ERplus_cohort1_predicted",
                                                        str_detect(., "ERplus") & str_detect(., "cohort2")& str_detect(., "\\(n") ~ "ERplus_cohort2_predicted",
                                                        str_detect(., "ERplus") & str_detect(., "cohort3")& str_detect(., "\\(n") ~ "ERplus_cohort3_predicted",
                                                        str_detect(., "ERplus") & str_detect(., "cohort4")& str_detect(., "\\(n") ~ "ERplus_cohort4_predicted",
                                                        str_detect(., "ERplus") & str_detect(., "cohort5")& str_detect(., "\\(n") ~ "ERplus_cohort5_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "cohort1")& str_detect(., "\\(n") ~ "ERminus_cohort1_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "cohort2")& str_detect(., "\\(n") ~ "ERminus_cohort2_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "cohort3")& str_detect(., "\\(n") ~ "ERminus_cohort3_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "cohort4")& str_detect(., "\\(n") ~ "ERminus_cohort4_predicted",
                                                        str_detect(., "ERminus") & str_detect(., "cohort5")& str_detect(., "\\(n") ~ "ERminus_cohort5_predicted",
                                                        TRUE ~ .)) %>%
  mutate(ERplus_cohort1_observed = ERplus_cohort1_predicted[Model == "METABRIC"],
         ERplus_cohort2_observed = ERplus_cohort2_predicted[Model == "METABRIC"],
         ERplus_cohort3_observed = ERplus_cohort3_predicted[Model == "METABRIC"],
         ERplus_cohort4_observed = ERplus_cohort4_predicted[Model == "METABRIC"],
         ERplus_cohort5_observed = ERplus_cohort5_predicted[Model == "METABRIC"],
         ERminus_cohort1_observed = ERminus_cohort1_predicted[Model == "METABRIC"],
         ERminus_cohort2_observed = ERminus_cohort2_predicted[Model == "METABRIC"],
         ERminus_cohort3_observed = ERminus_cohort3_predicted[Model == "METABRIC"],
         ERminus_cohort4_observed = ERminus_cohort4_predicted[Model == "METABRIC"],
         ERminus_cohort5_observed = ERminus_cohort5_predicted[Model == "METABRIC"],
  ) %>%
  pivot_longer(cols = -c(Model, contains("delta")), names_to = c("ERstatus", "cohort", "type"), names_sep = "_") %>%
  mutate(Model = as.factor(Model),
         delta = case_when(ERstatus == "ERplus" & cohort == "cohort1" ~ ERplus_cohort1_delta/100,
                           ERstatus == "ERplus" & cohort == "cohort2" ~ ERplus_cohort2_delta/100,
                           ERstatus == "ERplus" & cohort == "cohort3" ~ ERplus_cohort3_delta/100,
                           ERstatus == "ERplus" & cohort == "cohort4" ~ ERplus_cohort4_delta/100,
                           ERstatus == "ERplus" & cohort == "cohort5" ~ ERplus_cohort5_delta/100,
                           ERstatus == "ERminus" & cohort == "cohort1" ~ ERminus_cohort1_delta/100,
                           ERstatus == "ERminus" & cohort == "cohort2" ~ ERminus_cohort2_delta/100,
                           ERstatus == "ERminus" & cohort == "cohort3" ~ ERminus_cohort3_delta/100,
                           ERstatus == "ERminus" & cohort == "cohort4" ~ ERminus_cohort4_delta/100,
                           ERstatus == "ERminus" & cohort == "cohort5" ~ ERminus_cohort5_delta/100),
         value = value/100) %>%

  # Remove METABRIC
  filter(Model != "METABRIC") %>%
  mutate(Model = fct_relevel(Model, "ROR", "MP", "EP", "ODx", "PREDICT")) %>%
  dplyr::select(Model, ERstatus, cohort, type, value, delta) %>%
  mutate(Model = fct_recode(Model, "PREDICT\n + Prosigna" = "ROR", "PREDICT\n + MammaPrint" = "MP", "PREDICT\n + EndoPredict" = "EP",
                            "PREDICT\n + Oncotype Dx" = "ODx"))

# Produce dumbbell plots
for (i in 1:2) {
  iteration <- c("ERplus", "ERminus")

  for (j in 1:5) {

    cohortstat <- c("cohort1", "cohort2", "cohort3", "cohort4", "cohort5")

    er <- cohort %>%
      filter(ERstatus == iteration[i] & cohort == cohortstat[j])

    plot <- ggplot(
      data = er,
      mapping = aes(
        # Y Axis
        y=Model,
        # X Axis
        x=value,
        # If difference is zero, put it into a third group
        color = ifelse(delta == 0, "zero", type))) +
      # Plot lines between points, by model
      geom_line(
        mapping = aes(group = Model),
        color = gray,
        size = 2.5
      ) +
      geom_point(
        # Dot size
        size=4,
        # dot type
        pch = 19
      ) +
      # Add % for each point
      geom_text(
        # Font size
        size = 4,
        # Set text a little below the dots
        nudge_y = -0.35,
        nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
        mapping =
          aes(
            label =
              # If type is PREDICT (the first one), plot numbers with %
              ifelse(Model == "PREDICT",
                     # Value_if_True:
                     paste0(as.character(round(value*100,1)),"%"),
                     # Value_if_False
                     paste0(as.character(round(value*100,1)))
              ),
            color = ifelse(delta == 0, "zero", type)
          )
      ) +
      # Add "Legend" above PREDICT (the first one)
      geom_text(
        # Font size
        size = 4,
        # Bold face
        fontface = "bold",
        # Set text a little above the dots
        nudge_y = 0.6,
        nudge_x = ifelse(er$type == "predicted", -0.025, 0.025),
        mapping =
          aes(
            label =
              # If PREDICT, plot legend
              ifelse(Model == "PREDICT",
                     # Value_if_True:
                     ifelse(type == "predicted",
                            # Value_if_True:
                            "Predicted",
                            # Value_if_False:
                            "Observed"
                     ),
                     # Value_if_False
                     ""
              ),
            color = ifelse(delta == 0, "zero", type)
          )
      ) +
      # Change dot colors
      scale_color_manual(
        values = c(orange, green, "gray")
      ) +
      # Change scale x axis
      scale_x_continuous(
        # Set limits to 0 and 1.2
        limits = c(0,1.2),
        # Show tick marks at every 25%
        breaks = seq(0,1,.25),
        # Change scale to percent
        labels = scales::percent
      ) +
      # Expand y axis scale so that the legend can fit
      scale_y_discrete(
        expand = expand_scale(add=c(0.65,1))
      ) +
      # Add white rectangle to set the area where the values of the differences will be
      geom_rect(
        mapping = aes(xmin = 1.01, xmax = Inf , ymin = -Inf, ymax = Inf),
        fill = "white",
        color = "white"
      ) +
      # Add rectangle with correct background color for the differences
      geom_rect(
        mapping = aes(xmin = 1.05, xmax = 1.15 , ymin = -Inf, ymax = Inf),
        fill = background_diff,
        color = background_diff
      ) +
      # Add Differences values
      geom_text(
        # Bold face
        fontface = "bold",
        # Font size
        size = 4,
        # Font Color
        colour = "black",
        # Position
        mapping =
          aes(
            x = 1.1,
            y = Model,
            label =
              # To avoid duplicate values, plot empty text for the first group and
              # plot the difference only for the Ages_18_34 group.
              ifelse(type == "predicted",
                     # Value_if_True
                     "",
                     #Value_if_False
                     # If the difference is equal to zero, do not put any signal.
                     # Otherwise, if Positive, put the + sign on the front.
                     ifelse(delta == 0,
                            # Value_if_True:
                            paste0(as.character(round(delta*100,1))),
                            # Value_if_False
                            ifelse(delta > 0,
                                   # Value_if_True
                                   paste0("+",as.character(round(delta*100,1))),
                                   # Value_if_False
                                   paste0(as.character(round(delta*100,1)))
                            )
                     )
              )
          )
      ) +
      # Insert Title of Differences
      geom_text(
        fontface = "bold",
        size = 4,
        colour = "gray",
        # Set text a little above the dots
        nudge_y = 0.6,
        # Position
        mapping =
          aes(
            x = 1.1,
            y = Model,
            label =
              # If PREDICT, plot values
              ifelse(Model == "PREDICT",
                     # Value_if_True
                     "Δ",
                     #Value_if_False
                     ""
              )
          )
      ) +
      # Axis Labels
      labs(
        x = "",
        y = ""
      ) +
      # Change background, General font size, and other things
      theme(
        # Change font color and text for all text outside geom_text
        text = element_text(color = "#4e4d47", size = 14),

        # Country names in bold face
        axis.text.y = element_text(face = "bold"),

        # Add space between x axis text and plot
        axis.text.x = element_text(vjust = -0.75),

        # Do not show tick marks
        axis.ticks = element_blank(),

        # Delete original legend
        legend.position = "none",

        # White background
        panel.background = element_blank(),

        # Country (y Axis) Lines
        panel.grid.major.y = element_line(colour = light_gray, size = 1),

        # Change Title Font
        plot.title = element_text(face = "bold", size = 16))

    ggsave(plot = plot, filename = paste(iteration[i], cohortstat[j], "dumbbellplot.png", sep = "_"), width = 3043, height = 1165, units = "px")
  }
}

