# Initial importing & library----- 
load("METABRICExp.RData")

library(tidyverse)
library(genefu)
library(survival)
library(nhs.predict)
library(mclust)
library(survcomp)

# Load missing genes
load("BAG1METABRIC.RData") 
load("DHCR7METABRIC.RData")
load("IL8.CENPUMETABRIC.RData")
load("IL18METABRIC.RData")
load("PamMETABRIC.RData")

Exp <- as.data.frame(Exp) %>%
  
  # Add missing genes
  bind_rows(as.data.frame(BAG1), as.data.frame(DHCR7), as.data.frame(IL8.CENPU),
            as.data.frame(IL18), as.data.frame(Pam)) %>%
  
  # Coerce to tibble and save rownames as new variable
  as_tibble(., rownames = "gene_id")

Exp2 <- Exp %>%
  
  # Add mean gene expressions for NEFL
  summarise(across(.cols = where(is.numeric), ~mean(.x, na.rm = TRUE))) %>%
  
  # Put the gene names at the start
  bind_rows(Exp, .)  %>% 
  
  # Rename empty row name to NEFL
  mutate(gene_id = replace_na(gene_id, "NEFL"))

rm(Exp, BAG1, DHCR7, IL8.CENPU, IL18, Pam)

# Load in clinical outcomes
outcomes <- read_tsv("TableS6.txt") 

# Cleanup of variables & initial exploration----
glimpse(outcomes)

outcomes %>% 
  
  # Check to ensure that last followup status makes sense with death/deathbreast
  filter(DeathBreast == 1) %>% 
  group_by(Last.Followup.Status) %>% 
  summarise(count = n())

# Check that followup time is the same for patients without local or distant recurrence
outcomes %>%
  filter(LR == 0 & DR == 0) %>% 
  summarise(meanTLR = mean(TLR), meanTDR = mean(TDR))

outcomes <- outcomes %>%
  
  # Remove extraneous variables
  subset(select = c(-MATCHED.NORMAL.METABRIC.ID, -Cohort, 
                    -Breast.Tumour.Laterality, -Breast.Surgery,
                    -Date.Of.Diagnosis, -RT, -ER.Status, -NPI,
                    -Histological.Type, -HER2.SNP6, -iC10,
                    -ClaudinSubtype, -Complete.Rec.History,
                    -Cellularity, -Pam50Subtype, 
                    -Last.Followup.Status)) %>%
  
  # Rename columns 
  rename(Nodes = Lymph.Nodes.Positive,
         Dx_Age = Age.At.Diagnosis,
         Menopause = Inferred.Menopausal.State) %>%
  
  # Mutate ER/PR/HER2 expression into logicals
  mutate(ER.Expr = case_when(ER.Expr == "+" ~ 1,
                             ER.Expr == "-" ~ 0),
         PR.Expr = case_when(PR.Expr == "+" ~ 1,
                             PR.Expr == "-" ~ 0),
         Her2.Expr = case_when(Her2.Expr == "+" ~ 1,
                               Her2.Expr == "-" ~ 0),
         
         # Mutate all non-integer values of Size from cm to mm
         Size = case_when(Size %% 1 > 0 ~ Size * 10, 
                          TRUE ~ Size)) %>%
  
  # Mutate therapy status for later use in PREDICT
  mutate(HT = case_when(HT == "NO/NA" ~ 0,
                        TRUE ~ 1),
         CT = case_when(CT == "NO/NA" ~ 0,
                        (CT == "PACL"| CT == "TAXOID"| CT == "CAPE") ~ 3,
                        TRUE ~ 2))

outcomes %>%
  group_by(Grade, ER.Expr) %>%
  summarise(count = n(),
            mean_size = mean(Size, na.rm = TRUE),
            max_size = max(Size, na.rm = TRUE),
            mean_nodes = mean(Nodes),
            min_nodes = min(Nodes),
            max_nodes = max(Nodes))

# Explore scaling of sample nkis data
data(nkis)
rm(annot.nkis, demo.nkis)

# Transpose to create tidy data structure
as_tibble(t(data.nkis)) %>%
  summarise(min = min(., na.rm = TRUE),               
            max = max(., na.rm = TRUE),               
            mean = mean(colMeans(., na.rm = TRUE)))   

# Explore scaling of Exp2
as_tibble(Exp2) %>%
  dplyr::select((where(is.numeric))) %>%
  summarise(min = min(., na.rm = TRUE),               
            max = max(., na.rm = TRUE),               
            mean = mean(colMeans(., na.rm = TRUE)))   

# Rescale Exp2 to reflect range of data.nkis
rescaler <- function(x){
  (x - min(x, na.rm = TRUE))/(max(x, na.rm = TRUE) - min(x, na.rm = TRUE)) * 2
}

Exp2 <- Exp2 %>%
  dplyr::select(where(is.numeric)) %>%
  mutate(across(.cols = everything(), .fns = rescaler)) %>%
  add_column(gene_id = Exp2$gene_id, .before = TRUE)

rm(rescaler, data.nkis)

# PREDICT----

# Derive Ki67 from MKI67 gene expression
MKI67 <- Exp2 %>%
  
  # Pull MKI67 gene data
  filter(gene_id == "MKI67") %>%
  .[-1] %>%
  t(.)

# Classify into two groups (+ & -) based on MKI67 expression
ki67_cat <- mclust::Mclust(MKI67, G = 1:2) %>%
  
  # Pull out classification data
  .$classification %>%
  as_tibble(rownames = "METABRIC.ID") %>%
  mutate(Ki67 = case_when(value == 1 ~ 0,
                          value == 2 ~ 1),
         .keep = "unused")

# Join Ki67 category to outcomes data
outcomes <- left_join(outcomes, ki67_cat, by = "METABRIC.ID")

# Breakdown of Ki67 count by category
outcomes %>%
  group_by(Ki67) %>%
  summarise(count = n())

# Example analyses - assessing proportions by clinicopathological variables
outcomes %>%
  group_by(Ki67, Grade) %>%
  drop_na() %>%
  summarise(count = n())

# Produce PREDICT (pi)
outcomes <- outcomes %>%
  
  # Produce underlying variables
  mutate(age_m1 = case_when(ER.Expr == 1 ~ (Dx_Age/10)^-2 - 0.0287449295,
                            ER.Expr == 0 ~ Dx_Age - 56.3254902),
         age_b1 = case_when(ER.Expr == 1 ~ 34.53642,
                            ER.Expr == 0 ~ 0.0089827),
         age_m2 = case_when(ER.Expr == 1 ~ (Dx_Age/10)^-2 * log(Dx_Age/10) - 
                              0.0510121013,
                            ER.Expr == 0 ~ 0),
         age_b2 = case_when(ER.Expr == 1 ~ -34.20342,
                            ER.Expr == 0 ~ 0),
         size_m = case_when(ER.Expr == 1 ~ log(Size/100) + 1.545233938,
                            ER.Expr == 0 ~ (Size/100)^0.5 - 0.5090456276),
         size_b = case_when(ER.Expr == 1 ~ 0.7530729,
                            ER.Expr == 0 ~ 2.093446),
         node_m = case_when(ER.Expr == 1 ~ log((Nodes + 1)/10) + 1.387566896,
                            ER.Expr == 0 ~ log((Nodes + 1)/10) + 1.086916249),
         node_b = case_when(ER.Expr == 1 ~ 0.7060723,
                            ER.Expr == 0 ~ 0.6260541),
         grade_b = case_when(ER.Expr == 1 ~ 0.746655,
                             ER.Expr == 0 ~ 1.129091),
         grade_m = case_when(ER.Expr == 1 ~ Grade,
                             (ER.Expr == 0 & (Grade == 2 | Grade == 3)) ~ 1,
                             (ER.Expr == 0 & Grade == 1) ~ 0),
         her2_b = case_when(Her2.Expr == 1 ~ 0.2413,
                            Her2.Expr == 0 ~ -0.0762),
         ki67_b = case_when(Ki67 == 1 & ER.Expr == 1 ~ 0.14904,
                            Ki67 == 0 & ER.Expr == 1 ~ -0.11333),
         ki67_b = tidyr::replace_na(ki67_b, 0),
         Ki67 = tidyr::replace_na(Ki67, 9)) %>%
  
  # Calculate PREDICT value
  mutate(PREDICT = age_b1 * age_m1 + age_b2 * age_m2 + size_b * size_m +
           node_b * node_m + grade_b * grade_m + her2_b + ki67_b,
         .keep = "unused") 
# Screening status omitted as unavailable in METABRIC outcomes  

predict_fn <- function (data, year = 10, age.start, size, grade, nodes, 
                        er, her2, ki67, generation, horm) {
  age.start <- deparse(substitute(age.start))
  size <- deparse(substitute(size))
  grade <- deparse(substitute(grade))
  nodes <- deparse(substitute(nodes))
  er <- deparse(substitute(er))
  her2 <- deparse(substitute(her2))
  ki67 <- deparse(substitute(ki67))
  generation <- deparse(substitute(generation))
  horm <- deparse(substitute(horm))
  
  predict2.1s <- function(age.start, screen = 2, size, grade, nodes, 
                          er, her2, ki67, generation = 2, horm, 
                          traz = 0, bis = 0) {
    screen <- 0.204
    grade <- ifelse(grade == 9, 2.13, grade)
    time <- c(1:15)
    age <- age.start - 1 + time
    grade.val <- ifelse(er == 1, grade, ifelse(grade == 2 || grade == 3, 1, 0))
    age.mfp.1 <- ifelse(er == 1, (age.start/10)^-2 - 0.0287449295, 
                        age.start - 56.3254902)
    age.beta.1 <- ifelse(er == 1, 34.53642, 0.0089827)
    age.mfp.2 <- ifelse(er == 1, (age.start/10)^-2 * log(age.start/10) - 
                          0.0510121013, 0)
    age.beta.2 <- ifelse(er == 1, -34.20342, 0)
    size.mfp <- ifelse(er == 1, log(size/100) + 1.545233938, 
                       (size/100)^0.5 - 0.5090456276)
    size.beta <- ifelse(er == 1, 0.7530729, 2.093446)
    nodes.mfp <- ifelse(er == 1, log((nodes + 1)/10) + 1.387566896, 
                        log((nodes + 1)/10) + 1.086916249)
    nodes.beta <- ifelse(er == 1, 0.7060723, 0.6260541)
    grade.beta <- ifelse(er == 1, 0.746655, 1.129091)
    screen.beta <- 0
    her2.beta <- ifelse(her2 == 1, 0.2413, ifelse(her2 == 0, -0.0762, 0))
    ki67.beta <- ifelse(ki67 == 1 & er == 1, 0.14904, 
                        ifelse(ki67 == 0 & er == 1, -0.11333, 0))
    r.prop <- 0.69
    r.breast <- 0.82
    r.other <- 1.07
    mi <- 0.0698252 * ((age.start/10)^2 - 34.23391957)
    pi <- age.beta.1 * age.mfp.1 + age.beta.2 * age.mfp.2 + 
      size.beta * size.mfp + nodes.beta * nodes.mfp + grade.beta * 
      grade.val + screen.beta * screen + her2.beta + ki67.beta
    
    c <- -0.248 #assume all receive 2nd gen chemo
    h <- ifelse(horm == 1 & er == 1, -0.3857, 0)
    t <- ifelse(her2 == 1 & traz == 1, -0.3567, 0)
    b <- ifelse(bis == 1, -0.198, 0)
    hc <- h + c
    ht <- h + t
    hb <- h + b
    ct <- c + t
    cb <- c + b
    tb <- t + b
    hct <- h + c + t
    hcb <- h + c + b
    htb <- h + t + b
    ctb <- c + t + b
    hctb <- h + c + t + b
    base.m.cum.oth <- exp(-6.052919 + (1.079863 * log(time)) + 
                            (0.3255321 * time^0.5))
    s.cum.oth <- exp(-exp(mi) * base.m.cum.oth)
    base.m.oth <- base.m.cum.oth
    for (i in 2:15) {
      base.m.oth[i] <- base.m.cum.oth[i] - base.m.cum.oth[i - 
                                                            1]
    }
    rx.oth <- c(surg = 0, h = 0, c = 0, t = 0, b = 0, hc = 0, 
                ht = 0, hb = 0, ct = 0, cb = 0, tb = 0, hct = 0, 
                hcb = 0, htb = 0, ctb = 0, hctb = 0)
    cols <- length(rx.oth)
    m.oth.rx <- sapply(rx.oth, function(rx.oth, x.vector = base.m.oth) {
      output <- x.vector * exp(mi + rx.oth)
      return(output)
    })
    m.cum.oth.rx <- apply(m.oth.rx, 2, cumsum)
    s.cum.oth.rx <- exp(-m.cum.oth.rx)
    m.cum.oth.rx <- 1 - s.cum.oth.rx
    m.oth.rx <- m.cum.oth.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.oth.rx[i, j] <- m.cum.oth.rx[i, j] - m.cum.oth.rx[i - 
                                                              1, j]
      }
    }
    if (er == 1) {
      base.m.cum.br <- exp(0.7424402 - 7.527762/time^0.5 - 
                             1.812513 * log(time)/time^0.5)
    }
    else {
      base.m.cum.br <- exp(-1.156036 + 0.4707332/time^2 - 
                             3.51355/time)
    }
    base.m.br <- base.m.cum.br
    for (i in 2:15) {
      base.m.br[i] <- base.m.cum.br[i] - base.m.cum.br[i - 
                                                         1]
    }
    rx <- c(surg = 0, h = h, c = c, t = t, b = b, hc = hc, 
            ht = ht, hb = hb, ct = ct, cb = cb, tb = tb, hct = hct, 
            hcb = hcb, htb = htb, ctb = ctb, hctb = hctb)
    m.br.rx <- sapply(rx, function(rx, x.vector = base.m.br) {
      output <- x.vector * exp(pi + rx)
      return(output)
    })
    m.cum.br.rx <- apply(m.br.rx, 2, cumsum)
    s.cum.br.rx <- exp(-m.cum.br.rx)
    m.cum.br.rx <- 1 - s.cum.br.rx
    m.br.rx <- m.cum.br.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.br.rx[i, j] <- m.cum.br.rx[i, j] - m.cum.br.rx[i - 
                                                           1, j]
      }
    }
    m.cum.all.rx <- 1 - s.cum.oth.rx * s.cum.br.rx
    s.cum.all.rx <- 100 - 100 * m.cum.all.rx
    m.all.rx <- m.cum.all.rx
    for (j in 1:cols) {
      for (i in 2:15) {
        m.all.rx[i, j] <- m.cum.all.rx[i, j] - m.cum.all.rx[i - 
                                                              1, j]
      }
    }
    prop.br.rx <- m.br.rx/(m.br.rx + m.oth.rx)
    pred.m.br.rx <- prop.br.rx * m.all.rx
    pred.cum.br.rx <- apply(pred.m.br.rx, 2, cumsum)
    pred.m.oth.rx <- m.all.rx - pred.m.br.rx
    pred.cum.oth.rx <- apply(pred.m.oth.rx, 2, cumsum)
    pred.cum.all.rx <- pred.cum.br.rx + pred.cum.oth.rx
    benefits2.1 <- 100 * (pred.cum.all.rx[, 1] - pred.cum.all.rx)
    Overall <- round((1 - pred.cum.all.rx[year, 1]) * 100, 
                     2)
    horm.ben <- round(benefits2.1[year, 2], 2)
    chemo.ben <- round(benefits2.1[year, 6] - benefits2.1[year, 
                                                          2], 2)
    return(c(Overall, chemo.ben))
  }
  {
    N <- nrow(data)
    results <- array(c(NA, NA), c(N, 2))
    O.S <- c("PREDICT_OS")
    C.B <- c("PREDICT_deltaCT")
    colnames(results) <- c(O.S, C.B)
  }
  for (n in 1:N) {
    results[n, ] <- predict2.1s(data[[age.start]][n], data[[screen]][n], 
                                data[[size]][n], data[[grade]][n], data[[nodes]][n], 
                                data[[er]][n], data[[her2]][n], data[[ki67]][n], 
                                data[[generation]][n], data[[horm]][n])
  }
  cbind.data.frame(data, results)
}

outcomes <- predict_fn(outcomes, age.start = Dx_Age, size = Size,
                       grade = Grade, nodes = Nodes, 
                       er = ER.Expr, her2 = Her2.Expr,
                       ki67 = Ki67, generation = CT, horm = HT)

rm(predict_fn, ki67_cat, MKI67)

# OncotypeDx----
data("sig.oncotypedx")

odx_fn <- function(dataset) {
  
  data <- dataset %>%
    
    # Truncate dataset to include only ODx genes
    filter(gene_id %in% sig.oncotypedx$symbol) %>%
    dplyr::select(-gene_id) %>%
    
    # Apply normalisation technique from genefu to dataset
    apply(1, function(inputs) {
      xx <- (inputs - min(inputs, na.rm = TRUE))/(max(inputs, na.rm = TRUE) - 
                                                    min(inputs, na.rm = TRUE))
      return(xx * 15)
    })
  
  # Add row and colnames
  data <- data.frame(data)
  colnames(data) <- dataset$gene_id[dataset$gene_id %in% sig.oncotypedx$symbol]
  METABRIC.ID <- colnames(dataset)[-1]
  
  as_tibble(data, rownames = NA) %>%
    
    # Use weightings from model for each group
    mutate(grb7 = (0.9 * GRB7 + 0.1 * ERBB2),
           er = ((0.8 * ESR1 + 1.2 * PGR + BCL2 + SCUBE2)/4),
           prol = ((BIRC5 + MKI67 + MYBL2 + CCNB1 + AURKA)/5),
           inv = ((CTSL2 + MMP11)/2), 
           
           # Recode minimum contribution
           grb7 = case_when(grb7 < 8 ~ 8,
                            TRUE ~ grb7),
           prol = case_when(prol < 6.5 ~ 6.5,
                            TRUE ~ prol),
           
           # Calculate recurrence score
           rsu = (0.47 * grb7 - 0.34 * er + 1.04 * prol + 0.1 * inv +
                    0.05 * CD68 - 0.08 * GSTM1 - 0.07 * BAG1)
    ) %>%
    
    # Rescale recurrence score
    transmute(rsu = (20 * (rsu - 6.7))) %>%
    
    # Alter lower and upper limits
    mutate(ODx = case_when(rsu < 0 ~ 0,                # rsr min is 0
                           rsu > 100 ~ 100,            # rsr max is 100
                           TRUE ~ rsu),
           
           # Assign risk categories
           ODx_cat = case_when(ODx < 18 ~ "Low",
                               (ODx >= 18 & ODx <31) ~ "Int",
                               ODx >= 31 ~ "High"), 
           .keep = "none",                             # Drops all other vars
           
           # Add row names
           METABRIC.ID = METABRIC.ID)
}

# NB function input should be genes in rows, patients in cols
outcomes <- odx_fn(Exp2) %>%
  # Join with outcomes dataset
  right_join(outcomes, by = "METABRIC.ID")

rm(sig.oncotypedx, odx_fn)

# Checking that ODx values make sense
hist(outcomes$ODx)

outcomes %>% 
  group_by(ODx_cat) %>%              
  summarise(mean_ODx = mean(ODx),  
            min_ODx = min(ODx),    
            max_ODx = max(ODx))

# EPClin ----
data("sig.endoPredict")

# Calculate EP score
ep_fn <- function(dataset) {
  
  sig <- sig.endoPredict %>% 
    
    # Pull out relevant genes from signature
    filter(group == "GOI") %>%
    dplyr::select(weight, a, b)
  
  data <- dataset %>%
    
    # Truncate dataset to include only EP genes
    filter(gene_id %in% rownames(sig)) %>%
    
    # Apply normalisation technique from genefu to dataset
    mutate_if(is.numeric, ~(. - mean(., na.rm = TRUE) + log2(500))) 
  
  METABRIC.ID <- colnames(dataset)[-1]
  
  data <- data %>%
    
    # Perform normalisations per genefu
    bind_cols(sig[, c("a","b")]) %>%
    rowwise() %>%
    mutate_if(is.numeric, ~((. - b/a))) %>%
    dplyr::select(-a, -b) %>%
    
    bind_cols(sig[,c("weight", "a")]) %>%
    rowwise() %>%
    mutate_if(is.numeric, ~(. * weight)) %>%
    dplyr::select(-weight, -a) %>%
    
    # Calculate unscaled rs
    ungroup() %>%
    dplyr::select(where(is.numeric)) %>%
    summarise(rsu = colMeans(.) - 2.63) %>%
    
    # Perform rs scaling  
    mutate(rs = 1.5 * rsu + 8.57, # Redefine median at cutoff
           .keep = "none") %>%
    
    # Alter lower and upper limits
    mutate(EP = rs,
           # Assign risk categories
           EP_cat = case_when(rs < 5  ~ "Low",
                              rs >= 5 ~ "High"), 
           .keep = "none")  %>%                     # Drops all other vars
    
    # Rebind METABRIC.ID
    bind_cols(METABRIC.ID = METABRIC.ID)
}

outcomes <- ep_fn(Exp2) %>%
  right_join(outcomes, by = "METABRIC.ID")

rm(sig.endoPredict, ep_fn)

# Calculate EPClin score
epclin_fn <- function(dataset) {
  
  METABRIC.ID <- dataset %>%
    dplyr::select(METABRIC.ID)
  
  dataset %>%
    dplyr::select(Nodes, Size, EP) %>%
    
    # Recode tumour, nodes, score
    mutate(t = case_when(Size <= 10 ~ 1,
                         (Size > 10 & Size <= 20) ~ 2,
                         (Size > 20 & Size <= 50) ~ 3,
                         Size > 30 ~ 4),
           n = case_when(Nodes == 0 ~ 1,
                         (Nodes > 0 & Nodes <= 3) ~ 2,
                         (Nodes > 3 & Nodes <= 10) ~ 3,
                         Nodes > 10 ~ 4)
    ) %>%
    
    # Calculate EPClin score
    mutate(EPClin = (0.35 * t + 0.64 * n + 0.28 * EP),
           EPClin_cat = case_when(EPClin <= 3.3 ~ "Low",
                                  EPClin > 3.3 ~ "High"),
           .keep = "none") %>%
    
    # Rebind IDs
    bind_cols(METABRIC.ID = METABRIC.ID)
}

outcomes <- epclin_fn(outcomes) %>%
  right_join(outcomes, by = "METABRIC.ID")

rm(epclin_fn)

# Mammaprint----
data(sig.gene70)

# Manual editing of sig.gene70 to reflect dataset coding
sig.gene70 <- sig.gene70 %>% 
  mutate(gene_id = recode(NCBI.gene.symbol,
                          "ZNF533" = "ZNF385B",
                          "PECI" = "ECI2",
                          "LGP2" = "DHX58",
                          "AYTL2" = "LPCAT1",
                          "KNTC2" = "NDC80",
                          "QSCN6L1" = "QSOX2",
                          "ORC6" = "ORC6L"),
         .keep = "unused")

table(sig.gene70$gene_id %in% rownames(Exp2) | 
        sig.gene70$Alternative.symbols %in% rownames(Exp2))

mp_fn <- function (dataset) {
  
  data <- dataset %>%
    
    # Truncate dataset to include only MP genes
    filter(gene_id %in% sig.gene70$gene_id)
  
  # Save METABRIC.ID for later
  METABRIC.ID <- colnames(dataset)[-1]
  
  score <- Exp2 %>%
    
    # Join with signature to have correlation coefficients
    inner_join(sig.gene70, by = "gene_id") %>%
    
    # Correlation scores per patient
    summarise(across(where(is.numeric),
                     ~ cor(x = .x,
                           y = average.good.prognosis.profile,
                           method = "spearman")))
  
  # Identifying cutoffs and risks
  as_tibble(t(score), rownames = "METABRIC.ID") %>%
    mutate(MP = V1,
           MP_cat = case_when(score <= -0.3 ~ "Low",
                              score > -0.3 ~ "High"),
           .keep = "unused")
}

# Run Mammaprint function on outcomes
outcomes <- mp_fn(Exp2) %>%
  right_join(outcomes, by = "METABRIC.ID")

rm(sig.gene70, mp_fn)

# Prosigna ----

sig.pam50 <- read_tsv("Parker_centroids_corrected_genes.txt")

# Manual editing of sig.pam50 to reflect dataset coding
sig.pam50 <- sig.pam50 %>% 
  mutate(gene_id = recode(IlluminaID,
                          "ORC6L" = "ORC6",
                          "CDCA1" = "NUF2",
                          "DQ893928" = "NDC80",
                          "DQ894085" = "MYC",
                          "PGDH3" = "PGD",
                          "DQ895975" = "UBE2T",
                          "MIA" = "MIA-RAB4B...22666"),
         .keep = "unused")

# Ensure no values are missing from gene set
table(sig.pam50$gene_id %in% Exp2$gene_id)

pam50_fn <- function(dataset) {
  
  data <- dataset %>%
    dplyr::filter(gene_id %in% sig.pam50$gene_id)
  
  data %>%
    
    # Join pam50 signature to left of dataset 
    full_join(sig.pam50, ., by = "gene_id") %>%
    
    # Calculate correlations per PAM50 subtype
    summarise(across(c(-1:-6),          # Skip pam50 signature and gene_id
                     .fns = list(Basal = ~(cor(x = ., y = Basal, method = "spearman")),
                                 Her2 = ~(cor(x = ., y = Her2, method = "spearman")),
                                 LumA = ~(cor(x = ., y = LumA, method = "spearman")),
                                 LumB = ~(cor(x = ., y = LumB, method = "spearman")),
                                 Normal = ~(cor(x = ., y = Normal, method = "spearman"))
                     ))) %>%
    
    gather(key = "METABRIC.ID", value = "score") %>%
    separate(col = METABRIC.ID, sep = "_", into = c("METABRIC.ID", "type")) %>%
    
    group_by(METABRIC.ID) %>%
    
    # Store name of type with highest likelihood
    mutate(PAM50_cat = type[score == max(score)]) %>%
    
    # Create single dataframe with five scores per METABRIC.ID
    spread(key = "type", value = "score")
}

outcomes <- pam50_fn(Exp2) %>%
  right_join(outcomes, by = "METABRIC.ID")

rm(sig.pam50, pam50_fn)

# Calculate Prosigna ROR score

outcomes <- outcomes %>%
  
  # Produce ROR-S
  mutate(ROR = 0.05*Basal + 0.12*Her2 - 0.34*LumA +
           0.23*LumB)

rm(Exp2)

# Store outcomes as .csv output file
write_csv(outcomes, "brca_patient_outcomes.csv")
